import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _pt from "prop-types";
import React, { createRef, forwardRef, useEffect, useMemo, useState } from 'react';
import { MdKeyboardArrowRight, MdMoreHoriz } from 'react-icons/md';
import { Box, Flex } from '../Layout';
import { Dropdown } from '../Dropdown';
import { Ellipsis } from '../Ellipsis';
import { Icon } from '../Icon';
import { Position } from '../constants';
import { useMeasure } from '../hooks';
import { MoreIcon, StyledBreadcrumbLink } from './styles';
var BreadcrumbLink = /*#__PURE__*/forwardRef(function BreadcrumbLink(_ref, ref) {
  var active = _ref.active,
      name = _ref.name,
      lockWidth = _ref.lockWidth,
      onClick = _ref.onClick;
  return /*#__PURE__*/React.createElement(StyledBreadcrumbLink, {
    ref: ref,
    active: active,
    lockWidth: lockWidth,
    onClick: onClick
  }, /*#__PURE__*/React.createElement(Ellipsis, null, name));
});

var HiddenBreadcrumb = function HiddenBreadcrumb(_ref2) {
  var items = _ref2.items;
  return /*#__PURE__*/React.createElement(Dropdown, {
    position: Position.BOTTOM,
    overlay: /*#__PURE__*/React.createElement(Dropdown.List, null, items.map(function (_ref3) {
      var key = _ref3.key,
          name = _ref3.name,
          onClick = _ref3.onClick;
      return /*#__PURE__*/React.createElement(Dropdown.Item, {
        key: key,
        onClick: onClick
      }, name);
    }))
  }, /*#__PURE__*/React.createElement(MoreIcon, {
    type: MdMoreHoriz,
    fill: "gray400",
    cursor: "pointer"
  }));
};

HiddenBreadcrumb.propTypes = {
  items: _pt.arrayOf(_pt.shape({
    key: _pt.string.isRequired,
    name: _pt.node.isRequired,
    onClick: _pt.func
  })).isRequired
};
var Breadcrumb = /*#__PURE__*/forwardRef(function Breadcrumb(_ref4, forwardedRef) {
  var _ref4$items = _ref4.items,
      items = _ref4$items === void 0 ? [] : _ref4$items;

  var _useMeasure = useMeasure(forwardedRef),
      _useMeasure2 = _slicedToArray(_useMeasure, 2),
      ref = _useMeasure2[0].ref,
      bound = _useMeasure2[1];

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      measured = _useState2[0],
      setMeasured = _useState2[1];

  var _useState3 = useState(items.map(function (item) {
    return item.key;
  })),
      _useState4 = _slicedToArray(_useState3, 2),
      hiddenKeys = _useState4[0],
      setHiddenKeys = _useState4[1];

  var itemWithRefs = useMemo(function () {
    return items.map(function (attrs) {
      return _objectSpread(_objectSpread({}, attrs), {}, {
        itemRef: /*#__PURE__*/createRef()
      });
    });
  }, [items]);
  useEffect(function () {
    setMeasured(false);

    if (bound.width !== 0 && itemWithRefs.every(function (_ref5) {
      var _itemRef$current;

      var itemRef = _ref5.itemRef;
      return (_itemRef$current = itemRef.current) === null || _itemRef$current === void 0 ? void 0 : _itemRef$current.offsetWidth;
    })) {
      var containerWidth = bound.width - 48;

      var keys = _toConsumableArray(itemWithRefs).reverse().reduce(function (prev, _ref6, index) {
        var key = _ref6.key,
            itemRef = _ref6.itemRef;
        var hasArrow = index !== 0;
        containerWidth -= itemRef.current.offsetWidth + (hasArrow ? 24 : 0);

        if (containerWidth < 0) {
          return [].concat(_toConsumableArray(prev), [key]);
        }

        return prev;
      }, []);

      setHiddenKeys(keys);
      setMeasured(true);
    }
  }, [bound.width, itemWithRefs]);
  var hiddenItems = items.filter(function (_ref7) {
    var key = _ref7.key;
    return hiddenKeys.includes(key);
  });

  if (items.length === 0) {
    return null;
  }

  return /*#__PURE__*/React.createElement(Flex, {
    ref: ref,
    flex: "auto",
    alignItems: "center",
    overflow: "hidden",
    style: {
      opacity: measured ? 1 : 0,
      transition: 'transition: all 200ms ease 0s'
    }
  }, hiddenItems.length > 0 && /*#__PURE__*/React.createElement(HiddenBreadcrumb, {
    items: hiddenItems
  }), itemWithRefs.map(function (_ref8, index) {
    var key = _ref8.key,
        itemRef = _ref8.itemRef,
        name = _ref8.name,
        _ref8$onClick = _ref8.onClick,
        _onClick = _ref8$onClick === void 0 ? function () {} : _ref8$onClick;

    var hidden = hiddenKeys.includes(key);
    var active = index === itemWithRefs.length - 1;
    var arrow = index !== 0 && /*#__PURE__*/React.createElement(Icon, {
      type: MdKeyboardArrowRight,
      fill: "gray400"
    });
    return /*#__PURE__*/React.createElement(Box, {
      key: key,
      display: "inline-flex",
      overflow: "hidden",
      width: hidden ? 0 : 'auto'
    }, arrow, /*#__PURE__*/React.createElement(BreadcrumbLink, {
      ref: itemRef,
      active: active,
      name: name,
      lockWidth: items.length !== 1,
      onClick: function onClick(event) {
        if (!active) {
          _onClick(event);
        }
      }
    }));
  }));
});
export { Breadcrumb };