import styled, { css, keyframes } from 'styled-components';
import { space, width } from 'styled-system';
import { darken, lighten } from 'polished';
import { IconWrapper } from '../Icon';
export var spin = keyframes(["0%{transform:rotate(0deg);}100%{transform:rotate(360deg);}"]);

var buttonLoading = function buttonLoading(_ref) {
  var isLoading = _ref.isLoading,
      hasIcon = _ref.hasIcon;
  return isLoading && css(["opacity:0.5;cursor:default;pointer-events:none;&::before{width:1em;height:1em;margin-right:", "px;border:", ";border-radius:50%;border-top-color:transparent;border-right-color:transparent;animation:", " 0.7s linear infinite;}"], hasIcon ? 0 : 8, function (p) {
    return p.theme.borders.base;
  }, spin);
};

var buttonSize = css(["", ";"], function (_ref2) {
  var hasIcon = _ref2.hasIcon,
      _ref2$size = _ref2.size,
      size = _ref2$size === void 0 ? 'md' : _ref2$size,
      _ref2$theme = _ref2.theme,
      paddings = _ref2$theme.paddings,
      heights = _ref2$theme.heights,
      fontSizes = _ref2$theme.fontSizes;

  if (hasIcon) {
    return {
      sm: css(["width:", ";height:", ";"], heights.sm, heights.sm),
      md: css(["width:", ";height:", ";"], heights.base, heights.base),
      lg: css(["width:", ";height:", ";"], heights.lg, heights.lg)
    }[size];
  }

  return {
    sm: css(["height:", ";padding:0 ", ";font-size:", ";"], heights.sm, paddings.xs, fontSizes.sm),
    md: css(["height:", ";padding:0 ", ";font-size:", ";"], heights.base, paddings.sm, fontSizes.base),
    lg: css(["height:", ";padding:0 ", ";font-size:", ";"], heights.lg, paddings.md, fontSizes.lg)
  }[size];
});
export var buttonVariant = function buttonVariant(_ref3) {
  var variant = _ref3.variant;

  switch (variant) {
    default:
      return css(["border-color:", ";background-color:", ";color:", ";&::before{border-color:", ";}", " svg{fill:", ";}&:hover{border-color:", ";}&:active{border-color:", ";color:", ";", " svg{fill:", ";}}"], function (p) {
        return p.theme.colors.gray300;
      }, function (p) {
        return p.theme.colors.light;
      }, function (p) {
        return p.theme.colors.gray700;
      }, function (p) {
        return p.theme.colors.gray700;
      }, IconWrapper, function (p) {
        return p.theme.colors.gray400;
      }, function (p) {
        return p.theme.colors.primaryLight;
      }, function (p) {
        return p.theme.colors.primary;
      }, function (p) {
        return p.theme.colors.primary;
      }, IconWrapper, function (p) {
        return p.theme.colors.primary;
      });

    case 'primary':
      return css(["background-color:", ";color:", ";font-weight:bold;", " svg{fill:", ";}&:hover{background-color:", ";}&:active{background-color:", ";}"], function (p) {
        return p.theme.colors.primary;
      }, function (p) {
        return p.theme.colors.light;
      }, IconWrapper, function (p) {
        return p.theme.colors.light;
      }, function (p) {
        return lighten(0.03, p.theme.colors.primary);
      }, function (p) {
        return darken(0.03, p.theme.colors.primary);
      });

    case 'primary-invert':
      return css(["background-color:", ";color:", ";font-weight:bold;&::before{border-color:", ";}", " svg{fill:", ";}&:hover{color:", ";", " svg{fill:", ";}}&:active{background-color:", ";color:", ";", " svg{fill:", ";}}"], function (p) {
        return p.theme.colors.light;
      }, function (p) {
        return p.theme.colors.primary;
      }, function (p) {
        return p.theme.colors.primary;
      }, IconWrapper, function (p) {
        return p.theme.colors.primary;
      }, function (p) {
        return lighten(0.03, p.theme.colors.primary);
      }, IconWrapper, function (p) {
        return lighten(0.03, p.theme.colors.primary);
      }, function (p) {
        return p.theme.colors.gray200;
      }, function (p) {
        return darken(0.03, p.theme.colors.primary);
      }, IconWrapper, function (p) {
        return darken(0.03, p.theme.colors.primary);
      });

    case 'danger':
      return css(["background-color:", ";color:", ";font-weight:bold;", " svg{fill:", ";}&:hover{background-color:", ";}&:active{background-color:", ";}"], function (p) {
        return p.theme.colors.danger;
      }, function (p) {
        return p.theme.colors.light;
      }, IconWrapper, function (p) {
        return p.theme.colors.light;
      }, function (p) {
        return lighten(0.03, p.theme.colors.danger);
      }, function (p) {
        return darken(0.03, p.theme.colors.danger);
      });

    case 'danger-invert':
      return css(["background-color:", ";color:", ";font-weight:bold;&::before{border-color:", ";}", " svg{fill:", ";}&:hover{color:", ";", " svg{fill:", ";}}&:active{background-color:", ";color:", ";", " svg{fill:", ";}}"], function (p) {
        return p.theme.colors.light;
      }, function (p) {
        return p.theme.colors.danger;
      }, function (p) {
        return p.theme.colors.danger;
      }, IconWrapper, function (p) {
        return p.theme.colors.danger;
      }, function (p) {
        return lighten(0.03, p.theme.colors.danger);
      }, IconWrapper, function (p) {
        return lighten(0.03, p.theme.colors.danger);
      }, function (p) {
        return p.theme.colors.gray200;
      }, function (p) {
        return darken(0.03, p.theme.colors.danger);
      }, IconWrapper, function (p) {
        return darken(0.03, p.theme.colors.danger);
      });

    case 'regular':
      return css(["background-color:", ";color:", ";font-weight:bold;", " svg{fill:", ";}&:hover{background-color:", ";}&:active{background-color:", ";}"], function (p) {
        return p.theme.colors.primaryLight;
      }, function (p) {
        return p.theme.colors.light;
      }, IconWrapper, function (p) {
        return p.theme.colors.light;
      }, function (p) {
        return lighten(0.03, p.theme.colors.primaryLight);
      }, function (p) {
        return darken(0.03, p.theme.colors.primaryLight);
      });

    case 'normal':
      return css(["background-color:", ";color:", ";font-weight:bold;", " svg{fill:", ";}&:hover{background-color:", ";}&:active{background-color:", ";}"], function (p) {
        return p.theme.colors.surface;
      }, function (p) {
        return p.theme.colors.primaryLight;
      }, IconWrapper, function (p) {
        return p.theme.colors.primaryLight;
      }, function (p) {
        return lighten(0.03, p.theme.colors.surface);
      }, function (p) {
        return darken(0.03, p.theme.colors.surface);
      });
  }
};
export var StyledButton = styled.button.withConfig({
  componentId: "sc-1vyhu0b-0"
})(["display:inline-flex;position:relative;align-items:center;justify-content:center;overflow:hidden;border:", ";border-radius:", ";border-color:transparent;line-height:", ";text-decoration:none;vertical-align:middle;white-space:nowrap;cursor:pointer;user-select:none;&:focus{outline:0;}&:disabled,&[disabled]{opacity:0.85;background-color:", ";color:", ";cursor:default;pointer-events:none;", " svg{fill:", ";}}&::before{content:'';display:inline-block;position:relative;left:0;flex:none;width:0;height:0;margin-right:0;", ";}", ";", ";", " ", " ", ";", ";", ";"], function (p) {
  return p.theme.borders.base;
}, function (p) {
  return p.theme.radii.lg;
}, function (p) {
  return p.theme.lineHeight;
}, function (p) {
  return p.theme.colors.gray300;
}, function (p) {
  return p.theme.colors.gray500;
}, IconWrapper, function (p) {
  return p.theme.colors.gray500;
}, function (p) {
  return p.theme.transition;
}
/* sc-declaration */
, function (p) {
  return p.rounded && css(["border-radius:999px;"]);
}, function (p) {
  return p.theme.transition;
}
/* sc-declaration */
, buttonSize
/* sc-declaration */
, buttonVariant
/* sc-declaration */
, buttonLoading
/* sc-declaration */
, space, width);