import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _pt from "prop-types";
var _excluded = ["visible", "title", "footer", "placement", "width", "height", "closable", "maskClosable", "onClose", "children"];
import React, { useMemo } from 'react';
import { animated, to, useTransition } from '@react-spring/web';
import CloseButton from '../Modal/CloseButton';
import { Backdrop } from '../Backdrop';
import { ESC_KEY_CODE, useKeydown, usePreventBodyScroll } from '../hooks';
import { Flex } from '../Layout';
import { Heading } from '../Typography';
import { Portal } from '../Portal';
import { StackingOrder } from '../constants';
import { DrawerWrapper } from './styles';

var formatBreadth = function formatBreadth(breadth) {
  return typeof breadth === 'number' ? "".concat(breadth, "px") : breadth;
};

var getTransformAxis = function getTransformAxis(placement) {
  if (['top', 'bottom'].includes(placement)) {
    return 'translateY';
  }

  return 'translateX';
};

var getTransformBreadth = function getTransformBreadth(_ref) {
  var placement = _ref.placement,
      breadth = _ref.breadth;
  var prefix = ['left', 'top'].includes(placement) ? '-' : '';
  return "".concat(prefix).concat(breadth);
};

var getWrapperBreadth = function getWrapperBreadth(_ref2) {
  var placement = _ref2.placement,
      width = _ref2.width,
      height = _ref2.height;

  if (['top', 'bottom'].includes(placement)) {
    return formatBreadth(height);
  }

  return formatBreadth(width);
};

var Drawer = function Drawer(_ref3) {
  var visible = _ref3.visible,
      title = _ref3.title,
      footer = _ref3.footer,
      _ref3$placement = _ref3.placement,
      placement = _ref3$placement === void 0 ? 'right' : _ref3$placement,
      _ref3$width = _ref3.width,
      width = _ref3$width === void 0 ? 400 : _ref3$width,
      _ref3$height = _ref3.height,
      height = _ref3$height === void 0 ? 300 : _ref3$height,
      _ref3$closable = _ref3.closable,
      closable = _ref3$closable === void 0 ? true : _ref3$closable,
      _ref3$maskClosable = _ref3.maskClosable,
      maskClosable = _ref3$maskClosable === void 0 ? true : _ref3$maskClosable,
      onClose = _ref3.onClose,
      children = _ref3.children,
      otherProps = _objectWithoutProperties(_ref3, _excluded);

  var breadth = useMemo(function () {
    return getWrapperBreadth({
      placement: placement,
      width: width,
      height: height
    });
  }, [height, placement, width]);
  var transformAxis = useMemo(function () {
    return getTransformAxis(placement);
  }, [placement]);
  var transformBreadth = useMemo(function () {
    return getTransformBreadth({
      placement: placement,
      breadth: breadth
    });
  }, [breadth, placement]);
  usePreventBodyScroll(visible);
  var transition = useTransition(visible, {
    from: {
      offset: transformBreadth
    },
    enter: {
      offset: '0px'
    },
    leave: {
      offset: transformBreadth
    }
  });
  useKeydown({
    listening: visible,
    keyCode: ESC_KEY_CODE,
    onKeydown: onClose
  });
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Backdrop, {
    visible: visible,
    onClick: function onClick() {
      return maskClosable && onClose();
    }
  }), transition(function (style, item) {
    return item && /*#__PURE__*/React.createElement(Portal, {
      defaultOrder: StackingOrder.OVERLAY
    }, /*#__PURE__*/React.createElement(animated.div, {
      style: {
        transform: to(style.offset, function (offset) {
          return "".concat(transformAxis, "(").concat(offset, ")");
        })
      }
    }, /*#__PURE__*/React.createElement(DrawerWrapper, _objectSpread({
      placement: placement,
      breadth: breadth
    }, otherProps), title && /*#__PURE__*/React.createElement(Flex, {
      flex: "none",
      alignItems: "center",
      p: "4"
    }, /*#__PURE__*/React.createElement(Flex, {
      flex: "auto"
    }, /*#__PURE__*/React.createElement(Heading.H3, null, title)), closable && /*#__PURE__*/React.createElement(CloseButton, {
      onCancel: onClose
    })), /*#__PURE__*/React.createElement(Flex, {
      flex: "auto",
      flexDirection: "column",
      overflowY: "auto"
    }, children), footer && /*#__PURE__*/React.createElement(Flex, {
      justifyContent: "flex-end",
      bg: "surface",
      px: "4",
      py: "3"
    }, footer))));
  }));
};

Drawer.propTypes = {
  visible: _pt.bool.isRequired,
  onClose: _pt.func.isRequired,
  title: _pt.node,
  footer: _pt.node,
  closable: _pt.bool,
  maskClosable: _pt.bool,
  width: _pt.oneOfType([_pt.string, _pt.number]),
  height: _pt.oneOfType([_pt.string, _pt.number])
};
export { Drawer };