import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _pt from "prop-types";
var _excluded = ["value", "defaultValue", "suggestions", "onBlur", "onChange", "onMentionCreate", "disabled", "creatable", "highlightInvalid", "formatCreateText"];
import React, { useCallback, useEffect, useReducer, useRef } from 'react';
import { mergeEventProps } from '../utils';
import { useClickOutside } from '../hooks';
import Suggestions from './Suggestions';
import { Highlights, MentionWrapper, Textarea } from './styles';
import { getMentionCursor, getOverlayPosition } from './utils';
import { applyHighlights } from './highlight';
import { getCaretCoordinates } from './textarea-caret-position';
var placement = 'bottom';
var UP_ARROW = 38;
var DOWN_ARROW = 40;
var ENTER = 13;
var ESCAPE = 27;
var TAB = 9;

var initialState = function initialState(_ref) {
  var value = _ref.value,
      suggestions = _ref.suggestions;
  return {
    value: value,
    isFocus: false,
    height: 0,
    activeIndex: -1,
    caretPos: -1,
    dropdownVisible: false,
    filteredSuggestions: suggestions.map(function (suggestion) {
      return {
        type: 'option',
        value: suggestion
      };
    }),
    overlayPosition: {
      top: 0,
      left: 0
    }
  };
};

var reducer = function reducer(state, action) {
  switch (action.type) {
    case 'closeDropdown':
      return _objectSpread(_objectSpread({}, state), {}, {
        activeIndex: -1,
        dropdownVisible: false
      });

    case 'openDropdown':
      return _objectSpread(_objectSpread({}, state), {}, {
        activeIndex: state.activeIndex === -1 ? 0 : state.activeIndex,
        dropdownVisible: true
      }, action.payload);

    case 'selectSuggestion':
      return _objectSpread(_objectSpread({}, state), {}, {
        dropdownVisible: false,
        activeIndex: -1
      }, action.payload);

    case 'setPreviousItemActive':
      return _objectSpread(_objectSpread({}, state), {}, {
        activeIndex: state.activeIndex - 1 < 0 ? state.filteredSuggestions.length - 1 : state.activeIndex - 1
      });

    case 'setNextItemActive':
      return _objectSpread(_objectSpread({}, state), {}, {
        activeIndex: state.activeIndex + 1 <= state.filteredSuggestions.length - 1 ? state.activeIndex + 1 : 0
      });

    case 'updateValue':
      return _objectSpread(_objectSpread({}, state), {}, {
        value: action.payload
      });

    case 'updateHeight':
      return _objectSpread(_objectSpread({}, state), {}, {
        height: action.payload
      });

    case 'updateCaretPos':
      return _objectSpread(_objectSpread({}, state), {}, {
        caretPos: action.payload
      });

    case 'onFocus':
      return _objectSpread(_objectSpread({}, state), {}, {
        isFocus: true
      });

    case 'onBlur':
      return _objectSpread(_objectSpread({}, state), {}, {
        isFocus: false
      });

    default:
      throw new Error();
  }
};

var Mention = function Mention(_ref2) {
  var valueFromProps = _ref2.value,
      defaultValue = _ref2.defaultValue,
      suggestions = _ref2.suggestions,
      onBlur = _ref2.onBlur,
      onChange = _ref2.onChange,
      onMentionCreate = _ref2.onMentionCreate,
      disabled = _ref2.disabled,
      creatable = _ref2.creatable,
      highlightInvalid = _ref2.highlightInvalid,
      formatCreateText = _ref2.formatCreateText,
      props = _objectWithoutProperties(_ref2, _excluded);

  var mentionRef = useRef(null);
  var highlightRef = useRef(null);
  var suggestionsRef = useRef(null);
  var laf = useRef();
  var cursorMention = useRef({
    mention: null,
    searchValue: null,
    startPos: -1,
    endPos: -1
  });

  var _useReducer = useReducer(reducer, initialState({
    value: valueFromProps || defaultValue || '',
    suggestions: suggestions
  })),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var value = valueFromProps || state.value;
  useEffect(function () {
    if (mentionRef.current) {
      dispatch({
        type: 'updateHeight',
        payload: mentionRef.current.offsetHeight
      });
    }
  }, []);
  useEffect(function () {
    if (state.caretPos !== -1 && mentionRef.current) {
      mentionRef.current.focus();
      mentionRef.current.setSelectionRange(state.caretPos, state.caretPos);
      dispatch({
        type: 'updateCaretPos',
        payload: -1
      });
    }
  }, [state.caretPos]);
  useClickOutside({
    listening: state.isFocus,
    refs: [suggestionsRef, mentionRef],
    onClickOutside: function onClickOutside() {
      if (onBlur) {
        onBlur();
      }

      dispatch({
        type: 'onBlur'
      });
    }
  });
  var closeDropdown = useCallback(function () {
    if (laf.current) {
      cancelAnimationFrame(laf.current);
    }

    dispatch({
      type: 'closeDropdown'
    });
  }, []);
  var resetDropdown = useCallback(function () {
    if (!mentionRef.current) {
      return;
    }

    var cursor = getMentionCursor(mentionRef.current.value, mentionRef.current.selectionStart);
    cursorMention.current = cursor;

    if (cursor.mention !== null) {
      var filteredSuggestions = [].concat(_toConsumableArray(suggestions.filter(function (suggestion) {
        return suggestion.includes(cursor.searchValue);
      }).map(function (suggestion) {
        return {
          type: 'option',
          value: suggestion
        };
      })), _toConsumableArray(creatable && cursor.createValue !== '' && !suggestions.includes(cursor.createValue) ? [{
        type: 'create',
        value: cursor.createValue
      }] : []));

      if (filteredSuggestions.length === 0) {
        closeDropdown();
        return;
      }

      var coordinates = getCaretCoordinates(mentionRef.current, cursor.startPos - 2);
      var mentionRect = mentionRef.current.getBoundingClientRect();
      var overlayPosition = getOverlayPosition({
        coordinates: coordinates,
        mentionRect: mentionRect,
        target: mentionRef.current,
        placement: placement
      });
      dispatch({
        type: 'openDropdown',
        payload: {
          overlayPosition: overlayPosition,
          filteredSuggestions: filteredSuggestions
        }
      });
      laf.current = requestAnimationFrame(resetDropdown);
    } else {
      closeDropdown();
    }
  }, [closeDropdown, creatable, suggestions]);
  var selectSuggestion = useCallback(function (suggestion) {
    var _cursorMention$curren = cursorMention.current,
        startPos = _cursorMention$curren.startPos,
        endPos = _cursorMention$curren.endPos;
    var insertValue = "{{".concat(suggestion.value, "}}");
    var newValue = [value.slice(0, startPos - 2), insertValue, value.slice(endPos + (value[endPos] === '}' ? 2 : 0), value.length)].join('');
    dispatch({
      type: 'selectSuggestion',
      payload: {
        caretPos: startPos - 2 + insertValue.length,
        value: newValue
      }
    });

    if (suggestion.type === 'create') {
      onMentionCreate(suggestion.value);
    }

    if (onChange) {
      onChange(newValue);
    }
  }, [onChange, onMentionCreate, value]);
  var handleKeyDown = useCallback(function (event) {
    var dropdownVisible = state.dropdownVisible,
        activeIndex = state.activeIndex,
        filteredSuggestions = state.filteredSuggestions;
    var keyCode = event.keyCode;

    if (dropdownVisible && keyCode === ENTER && activeIndex !== -1 && filteredSuggestions.length && cursorMention.current) {
      event.preventDefault();
      event.stopPropagation();
      selectSuggestion(filteredSuggestions[activeIndex]);
      return;
    }

    if (dropdownVisible && [TAB, ESCAPE].includes(keyCode)) {
      closeDropdown();
      return;
    }

    if (dropdownVisible && keyCode === UP_ARROW) {
      event.preventDefault();
      event.stopPropagation();
      dispatch({
        type: 'setPreviousItemActive'
      });
      return;
    }

    if (dropdownVisible && keyCode === DOWN_ARROW) {
      event.preventDefault();
      event.stopPropagation();
      dispatch({
        type: 'setNextItemActive'
      });
    }
  }, [closeDropdown, selectSuggestion, state]);
  var handleChange = useCallback(function (event) {
    var newValue = event.currentTarget.value;

    if (newValue !== value) {
      dispatch({
        type: 'updateValue',
        payload: newValue
      });

      if (onChange) {
        onChange(newValue);
      }
    }
  }, [onChange, value]);
  var handleOnFocus = useCallback(function () {
    dispatch({
      type: 'onFocus'
    });
  }, []);
  var composedProps = mergeEventProps(props, {
    onKeyDown: handleKeyDown,
    onChange: handleChange,
    onSelect: function onSelect() {
      if (!state.dropdownVisible) {
        resetDropdown();
      }
    },
    onScroll: function onScroll(event) {
      if (highlightRef.current) {
        highlightRef.current.scrollTop = event.currentTarget.scrollTop;
      }

      if (state.dropdownVisible) {
        closeDropdown();
      }
    },
    onResize: function onResize(event) {
      // https://github.com/buildo/react-autosize-textarea/issues/109#issuecomment-430002058
      setTimeout(function () {
        return dispatch({
          type: 'updateHeight',
          payload: event.target.offsetHeight
        });
      });
    },
    onFocus: handleOnFocus
  });
  return /*#__PURE__*/React.createElement(MentionWrapper, {
    disabled: disabled
  }, /*#__PURE__*/React.createElement(Highlights, {
    ref: highlightRef,
    style: {
      height: state.height
    },
    dangerouslySetInnerHTML: {
      __html: applyHighlights({
        suggestions: suggestions,
        value: value,
        highlightInvalid: highlightInvalid
      })
    }
  }), /*#__PURE__*/React.createElement(Textarea, _objectSpread(_objectSpread({
    ref: mentionRef,
    value: value,
    disabled: disabled
  }, composedProps), {}, {
    onBlur: function onBlur(event) {
      event.preventDefault();
      event.stopPropagation();
      closeDropdown();
    }
  })), /*#__PURE__*/React.createElement(Suggestions, {
    ref: suggestionsRef,
    formatCreateText: formatCreateText,
    dropdownVisible: state.dropdownVisible,
    overlayPosition: state.overlayPosition,
    filteredSuggestions: state.filteredSuggestions,
    activeIndex: state.activeIndex,
    onSuggestionClick: selectSuggestion
  }));
};

Mention.propTypes = {
  value: _pt.string,
  defaultValue: _pt.string,
  suggestions: _pt.arrayOf(_pt.string),
  disabled: _pt.bool,
  creatable: _pt.bool,
  highlightInvalid: _pt.bool,
  formatCreateText: _pt.func,
  onBlur: _pt.func,
  onChange: _pt.func,
  onMentionCreate: _pt.func
};
Mention.defaultProps = {
  defaultValue: '',
  suggestions: [],
  creatable: false,
  highlightInvalid: false,
  onMentionCreate: function onMentionCreate() {},
  formatCreateText: function formatCreateText(createValue) {
    return "Press Enter to create mention: ".concat(createValue);
  }
};
export { Mention };