import React, { forwardRef, useEffect } from 'react';
import { config, useTransition } from '@react-spring/web';
import { Portal } from '../Portal';
import { SuggestionItem, SuggestionList } from './styles';
var Suggestions = /*#__PURE__*/forwardRef(function Suggestions(_ref, suggestionRef) {
  var dropdownVisible = _ref.dropdownVisible,
      activeIndex = _ref.activeIndex,
      formatCreateText = _ref.formatCreateText,
      overlayPosition = _ref.overlayPosition,
      filteredSuggestions = _ref.filteredSuggestions,
      onSuggestionClick = _ref.onSuggestionClick;
  var translations = useTransition(dropdownVisible, {
    from: {
      opacity: 0
    },
    enter: {
      opacity: 1
    },
    leave: {
      opacity: 0
    },
    config: config.stiff
  });
  useEffect(function () {
    var _ref2 = suggestionRef,
        current = _ref2.current;

    if (current) {
      var activeItemScrollTop = activeIndex * 36;

      if (activeItemScrollTop + 36 >= current.scrollTop + current.offsetHeight) {
        current.scrollTop = activeItemScrollTop + 36 - current.offsetHeight;
      } else if (activeItemScrollTop <= current.scrollTop) {
        current.scrollTop = activeItemScrollTop;
      }
    }
  }, [activeIndex, suggestionRef]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, translations(function (style, item) {
    return item && /*#__PURE__*/React.createElement(Portal, null, /*#__PURE__*/React.createElement("div", {
      style: {
        willChange: 'transform',
        transform: "translate3d(".concat(overlayPosition.left, "px, ").concat(overlayPosition.top, "px, 0px)")
      }
    }, /*#__PURE__*/React.createElement(SuggestionList, {
      ref: suggestionRef,
      style: style
    }, filteredSuggestions.map(function (suggestion, index) {
      return /*#__PURE__*/React.createElement(SuggestionItem, {
        key: suggestion.value,
        active: index === activeIndex,
        onClick: function onClick() {
          return onSuggestionClick(suggestion);
        }
      }, suggestion.type === 'create' ? formatCreateText(suggestion.value) : suggestion.value);
    }))));
  }));
});
export default Suggestions;