export var getOverlayPosition = function getOverlayPosition(_ref) {
  var mentionRect = _ref.mentionRect,
      coordinates = _ref.coordinates,
      target = _ref.target,
      placement = _ref.placement;
  var top = mentionRect.top + coordinates.top - target.scrollTop + (placement === 'bottom' ? coordinates.height : 0);
  var left = mentionRect.left + coordinates.left - target.scrollLeft;
  return {
    top: top,
    left: left
  };
};
var PREFIX = '{{';
var SUFFIX = '}}';
export var getCursorPosition = function getCursorPosition(value, selectionStart) {
  var startPos = value.lastIndexOf(PREFIX, selectionStart) > -1 ? value.lastIndexOf(PREFIX, selectionStart) + 2 : -1;
  var suffixPos = value.indexOf(SUFFIX, selectionStart);
  var nextPrefixPos = value.indexOf(PREFIX, selectionStart);
  var spacePos = value.indexOf(' ', selectionStart);
  var positions = [suffixPos, nextPrefixPos, spacePos].filter(function (pos) {
    return pos !== -1;
  }).sort(function (a, b) {
    return a - b;
  });

  if (selectionStart === startPos && nextPrefixPos !== -1 && nextPrefixPos === positions[0]) {
    return {
      startPos: startPos,
      endPos: startPos
    };
  }

  var endPos = positions[0] || selectionStart;
  return {
    startPos: startPos,
    endPos: endPos
  };
};
export var getMentionCursor = function getMentionCursor(originValue, selectionStart) {
  var value = originValue.replace(/[\r\n]/g, ' ');

  var _getCursorPosition = getCursorPosition(value, selectionStart),
      startPos = _getCursorPosition.startPos,
      endPos = _getCursorPosition.endPos;

  var mention = value.substring(startPos, endPos);
  var searchValue = value.substring(startPos, selectionStart);
  var createValue = mention.length > searchValue.length ? mention : searchValue;

  if (startPos < 0 || endPos < 0 || mention.includes('{') || mention.includes('}') || selectionStart < startPos || selectionStart > endPos) {
    return {
      mention: null,
      searchValue: null,
      createValue: null,
      startPos: -1,
      endPos: -1
    };
  }

  return {
    mention: mention,
    searchValue: searchValue,
    createValue: createValue,
    startPos: startPos,
    endPos: endPos
  };
};