import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _pt from "prop-types";
var _excluded = ["children", "visible", "onCancel", "onOpenComplete", "onCloseComplete", "closable", "size", "status", "zIndex"];
import React from 'react';
import { animated, config, useChain, useSpring, useSpringRef, useTransition } from '@react-spring/web';
import { Backdrop } from '../Backdrop';
import { ESC_KEY_CODE, useKeydown } from '../hooks';
import { Portal } from '../Portal';
import { StackingOrder } from '../constants';
import { ModalContent, ModalStatusBar, ModalWrapper } from './styles';
var AnimatedModalWrapper = animated(ModalWrapper);
var AnimatedModalStatusBar = animated(ModalStatusBar);

var AnimationModal = function AnimationModal(_ref) {
  var _ref$children = _ref.children,
      children = _ref$children === void 0 ? '' : _ref$children,
      visible = _ref.visible,
      onCancel = _ref.onCancel,
      onOpenComplete = _ref.onOpenComplete,
      onCloseComplete = _ref.onCloseComplete,
      _ref$closable = _ref.closable,
      closable = _ref$closable === void 0 ? true : _ref$closable,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? 'md' : _ref$size,
      _ref$status = _ref.status,
      status = _ref$status === void 0 ? null : _ref$status,
      _ref$zIndex = _ref.zIndex,
      zIndex = _ref$zIndex === void 0 ? StackingOrder.OVERLAY : _ref$zIndex,
      otherProps = _objectWithoutProperties(_ref, _excluded);

  useKeydown({
    listening: closable ? visible : false,
    keyCode: ESC_KEY_CODE,
    onKeydown: onCancel
  });
  var transRef = useSpringRef();
  var transitions = useTransition(visible, {
    ref: transRef,
    from: {
      opacity: 0,
      transform: 'translate(-50%, -50%) scale(0.9)',
      pointerEvents: 'auto'
    },
    enter: {
      opacity: 1,
      transform: 'translate(-50%, -50%) scale(1)',
      pointerEvents: 'auto'
    },
    leave: {
      opacity: 0,
      transform: 'translate(-50%, -50%) scale(0.9)',
      pointerEvents: 'none'
    },
    onDestroyed: function onDestroyed(isDestroyed) {
      if (isDestroyed && onCloseComplete && !visible) {
        onCloseComplete();
      } else if (!isDestroyed && onOpenComplete && visible) {
        onOpenComplete();
      }
    },
    config: config.stiff
  });
  var springRef = useSpringRef();
  var statusProps = useSpring({
    ref: springRef,
    opacity: visible ? 1 : 0,
    top: visible ? -10 : 0
  });
  useChain(visible ? [transRef, springRef] : [springRef, transRef], [0, visible ? 0.15 : 0]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Backdrop, {
    visible: visible,
    zIndex: zIndex,
    onClick: function onClick(event) {
      if (closable) {
        onCancel(event);
      }
    }
  }), transitions(function (style, item) {
    return item && /*#__PURE__*/React.createElement(Portal, {
      defaultOrder: zIndex
    }, /*#__PURE__*/React.createElement(AnimatedModalWrapper, {
      style: style
    }, status && /*#__PURE__*/React.createElement(AnimatedModalStatusBar, {
      status: status,
      style: statusProps
    }), /*#__PURE__*/React.createElement(ModalContent, _objectSpread({
      size: size
    }, otherProps), children)));
  }));
};

AnimationModal.propTypes = {
  onCancel: _pt.oneOfType([_pt.func, _pt.func]).isRequired,
  onOpenComplete: _pt.func,
  onCloseComplete: _pt.func,
  closable: _pt.bool,
  visible: _pt.bool.isRequired,
  zIndex: _pt.number
};
export default AnimationModal;