import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _pt from "prop-types";
import React, { useCallback, useState } from 'react';
import { useIsomorphicLayoutEffect } from '@reach/utils';
import { useLocale } from '../locale';
import { Modal } from './Modal';

var HooksModal = function HooksModal(_ref) {
  var modalTriggerRef = _ref.modalTriggerRef;

  var _useLocale = useLocale(),
      locale = _useLocale.locale;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      visible = _useState2[0],
      setVisible = _useState2[1];

  var _useState3 = useState({
    type: 'confirm',
    closable: true,
    title: '',
    content: '',
    confirmText: '',
    cancelText: '',
    onConfirm: function onConfirm() {},
    onCancel: function onCancel() {},
    onOpenComplete: function onOpenComplete() {},
    onCloseComplete: function onCloseComplete() {}
  }),
      _useState4 = _slicedToArray(_useState3, 2),
      modalOptions = _useState4[0],
      setModalOptions = _useState4[1];

  var trigger = useCallback(function (options, type) {
    var _options$closable = options.closable,
        closable = _options$closable === void 0 ? type === 'confirm' : _options$closable,
        _options$title = options.title,
        title = _options$title === void 0 ? '' : _options$title,
        _options$content = options.content,
        content = _options$content === void 0 ? '' : _options$content,
        _options$confirmText = options.confirmText,
        confirmText = _options$confirmText === void 0 ? locale.Modal.confirmText : _options$confirmText,
        _options$cancelText = options.cancelText,
        cancelText = _options$cancelText === void 0 ? locale.Modal.cancelText : _options$cancelText,
        onConfirm = options.onConfirm,
        onCancel = options.onCancel,
        onOpenComplete = options.onOpenComplete,
        onCloseComplete = options.onCloseComplete,
        zIndex = options.zIndex;

    var resolveFn = function resolveFn() {};

    var confirmation = new Promise(function (resolve) {
      resolveFn = resolve;
    });

    var handleConfirm = function handleConfirm(event) {
      setVisible(false);

      if (onConfirm) {
        onConfirm(event);
      } else {
        resolveFn(true);
      }
    };

    var handleCancel = function handleCancel(event) {
      setVisible(false);

      if (onCancel) {
        onCancel(event);
      } else {
        resolveFn(false);
      }
    };

    var handleClose = function handleClose() {
      return handleCancel();
    };

    var handleUpdate = function handleUpdate(updateOptions) {
      return setModalOptions(function (prev) {
        return _objectSpread(_objectSpread({}, prev), updateOptions);
      });
    };

    setModalOptions({
      type: type,
      closable: closable,
      title: title,
      content: content,
      confirmText: confirmText,
      cancelText: cancelText,
      onConfirm: handleConfirm,
      onCancel: handleCancel,
      onOpenComplete: onOpenComplete,
      onCloseComplete: onCloseComplete,
      zIndex: zIndex
    });
    setVisible(true);
    var rtn = [confirmation, handleClose, handleUpdate];
    rtn.confirmation = confirmation;
    rtn.close = handleClose;
    rtn.update = handleUpdate;
    return rtn;
  }, [locale.Modal.cancelText, locale.Modal.confirmText]);
  var title = modalOptions.title,
      closable = modalOptions.closable,
      content = modalOptions.content,
      cancelText = modalOptions.cancelText,
      confirmText = modalOptions.confirmText,
      onCancel = modalOptions.onCancel,
      onConfirm = modalOptions.onConfirm,
      type = modalOptions.type,
      zIndex = modalOptions.zIndex;
  var status = type !== 'confirm' ? type : null;
  useIsomorphicLayoutEffect(function () {
    // eslint-disable-next-line no-param-reassign
    modalTriggerRef.current = trigger;
  }, [modalTriggerRef, trigger]);
  return /*#__PURE__*/React.createElement(Modal, {
    visible: visible,
    closable: closable,
    title: title,
    status: status,
    cancelText: cancelText,
    confirmText: confirmText,
    onCancel: onCancel,
    onConfirm: onConfirm,
    zIndex: zIndex,
    confirmButtonProps: {
      variant: type === 'error' ? 'danger' : 'primary'
    }
  }, content);
};

export default HooksModal;