import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _pt from "prop-types";
var _excluded = ["children", "position", "title", "content", "defaultVisible", "visible", "onVisibleChange", "onOpenComplete", "onCloseComplete", "Wrapper"];
import React, { createContext, useContext, useRef, useState } from 'react';
import { Position } from '../constants';
import { Positioner } from '../Positioner';
import { useClickOutside, useKeydown, useRenderChildren, useTargetRef, useToggleTrigger } from '../hooks';
import PopoverPopup from './PopoverPopup';
export var ClickOutsideContext = /*#__PURE__*/createContext({
  setHasChild: function setHasChild() {}
});
ClickOutsideContext.displayName = 'ClickOutsideContext';

var Popover = function Popover(_ref) {
  var children = _ref.children,
      _ref$position = _ref.position,
      position = _ref$position === void 0 ? Position.TOP : _ref$position,
      title = _ref.title,
      content = _ref.content,
      _ref$defaultVisible = _ref.defaultVisible,
      defaultVisible = _ref$defaultVisible === void 0 ? false : _ref$defaultVisible,
      visibleFromProps = _ref.visible,
      _onVisibleChange = _ref.onVisibleChange,
      onOpenComplete = _ref.onOpenComplete,
      onCloseComplete = _ref.onCloseComplete,
      Wrapper = _ref.Wrapper,
      otherProps = _objectWithoutProperties(_ref, _excluded);

  var popupRef = useRef(null);
  var targetRef = useTargetRef({
    children: children
  });

  var _useContext = useContext(ClickOutsideContext),
      setHasChildFromContext = _useContext.setHasChild;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      hasChild = _useState2[0],
      setHasChild = _useState2[1];

  var _useToggleTrigger = useToggleTrigger({
    visible: visibleFromProps,
    defaultVisible: defaultVisible,
    onVisibleChange: function onVisibleChange(newVisible) {
      setHasChildFromContext(newVisible);

      if (_onVisibleChange) {
        _onVisibleChange(newVisible);
      }
    }
  }),
      visible = _useToggleTrigger.visible,
      handleClose = _useToggleTrigger.handleClose,
      toggle = _useToggleTrigger.toggle;

  useClickOutside({
    listening: hasChild ? false : visible,
    refs: [targetRef, popupRef],
    onClickOutside: handleClose
  });
  useKeydown({
    listening: visible,
    keyCode: 27,
    onKeydown: handleClose
  });
  var renderChildren = useRenderChildren({
    targetRef: targetRef,
    children: children,
    mergeProps: {
      onClick: toggle
    }
  });
  return /*#__PURE__*/React.createElement(ClickOutsideContext.Provider, {
    value: {
      setHasChild: setHasChild
    }
  }, /*#__PURE__*/React.createElement(Positioner, {
    positionerRef: popupRef,
    targetRef: targetRef,
    visible: visible,
    onOpenComplete: onOpenComplete,
    onCloseComplete: onCloseComplete,
    position: position,
    positioner: /*#__PURE__*/React.createElement(PopoverPopup, _objectSpread({
      title: title,
      content: content,
      handleClose: handleClose,
      Wrapper: Wrapper
    }, otherProps))
  }, renderChildren));
};

Popover.propTypes = {
  Wrapper: _pt.elementType,
  defaultVisible: _pt.bool,
  visible: _pt.bool,
  onVisibleChange: _pt.func,
  title: _pt.oneOfType([_pt.node, _pt.func]),
  content: _pt.oneOfType([_pt.node, _pt.func]).isRequired,
  onOpenComplete: _pt.func,
  onCloseComplete: _pt.func
};
export { Popover };