import _pt from "prop-types";
import React, { useCallback, useEffect, useRef } from 'react';
import { createPortal } from 'react-dom';
import { Stack } from '../Stack';
var portalContainer;

var canUseDom = function canUseDom() {
  return Boolean(typeof window !== 'undefined' && window.document && window.document.createElement);
};

var createMountNode = function createMountNode(zIndex) {
  var element = document.createElement('div');
  element.style.position = 'fixed';
  element.style.top = '0';
  element.style.left = '0';
  element.style.zIndex = String(zIndex);
  return element;
};

var Portal = function Portal(_ref) {
  var defaultOrder = _ref.defaultOrder,
      children = _ref.children;
  var elementRef = useRef();
  useEffect(function () {
    return function () {
      if (elementRef.current) {
        portalContainer.removeChild(elementRef.current);
      }
    };
  }, []);
  var getElement = useCallback(function (zIndex) {
    if (elementRef.current) {
      return elementRef.current;
    }

    if (!portalContainer) {
      portalContainer = document.createElement('div');
      portalContainer.setAttribute('portal-container', '');
      document.body.appendChild(portalContainer);
    }

    elementRef.current = createMountNode(zIndex || 5);
    portalContainer.appendChild(elementRef.current);
    return elementRef.current;
  }, []); // This fixes SSR

  if (!canUseDom()) {
    return null;
  }

  return /*#__PURE__*/React.createElement(Stack, {
    defaultOrder: defaultOrder
  }, function (stackingOrder) {
    return /*#__PURE__*/createPortal(children, getElement(stackingOrder));
  });
};

Portal.propTypes = {
  defaultOrder: _pt.number
};
export { Portal };