import _pt from "prop-types";
import React, { useEffect, useRef } from 'react';
import { useTransition } from '@react-spring/web';
import { Portal } from '../Portal';
import { Position, StackingOrder } from '../constants';
import PositionerImpl from './PositionerImpl';

var Positioner = function Positioner(_ref) {
  var visible = _ref.visible,
      targetRefFromProps = _ref.targetRef,
      positionerRef = _ref.positionerRef,
      _ref$position = _ref.position,
      position = _ref$position === void 0 ? Position.TOP_RIGHT : _ref$position,
      children = _ref.children,
      positioner = _ref.positioner,
      onOpenComplete = _ref.onOpenComplete,
      onCloseComplete = _ref.onCloseComplete;
  var targetRefFromSelf = useRef(null);
  var targetRef = targetRefFromProps || targetRefFromSelf;
  useEffect(function () {
    var originalWarn = console.warn;

    console.warn = function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      if (/You need to place the ref/.test(args[0])) {
        return;
      }

      originalWarn.call.apply(originalWarn, [console].concat(args));
    };

    return function () {
      console.warn = originalWarn;
    };
  }, []);
  var transition = useTransition(visible, {
    keys: function keys(value) {
      return value ? 'on' : 'off';
    },
    from: {
      opacity: 0,
      transform: 'scale(0.9)'
    },
    enter: {
      opacity: 1,
      transform: 'scale(1)'
    },
    leave: {
      opacity: 0,
      transform: 'scale(1)',
      pointerEvents: 'none'
    },
    onDestroyed: function onDestroyed(isDestroyed) {
      if (isDestroyed) {
        if (!visible && onCloseComplete) {
          onCloseComplete();
        }
      } else if (visible && onOpenComplete) {
        if (onOpenComplete) {
          onOpenComplete();
        }
      }
    },
    config: {
      mass: 1,
      tension: 500,
      friction: 40
    }
  });
  return /*#__PURE__*/React.createElement(React.Fragment, null, children instanceof Function ? children({
    ref: targetRef
  }) : children, transition(function (style, item) {
    return item && /*#__PURE__*/React.createElement(Portal, {
      defaultOrder: StackingOrder.POSITIONER
    }, /*#__PURE__*/React.createElement(PositionerImpl, {
      style: style,
      positioner: positioner,
      position: position,
      targetRef: targetRef,
      positionerRef: positionerRef
    }));
  }));
};

Positioner.propTypes = {
  visible: _pt.bool,
  targetOffset: _pt.number,
  bodyOffset: _pt.number,
  onOpenComplete: _pt.func,
  onCloseComplete: _pt.func
};
export { Positioner };