import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _pt from "prop-types";
import React, { useMemo, useRef } from 'react';
import styled from 'styled-components';
import { animated } from '@react-spring/web';
import { useRect } from '@reach/rect';
import getPosition from './getPosition';
var StyledPositionerWrapper = styled.div.withConfig({
  componentId: "sc-12izl5s-0"
})(["position:absolute;top:0;left:0;will-change:transform;"]);
var targetOffset = 6;
var bodyOffset = 6;

var getPositionerProps = function getPositionerProps(_ref) {
  var position = _ref.position,
      positionerRef = _ref.positionerRef,
      positionerRect = _ref.positionerRect,
      targetRect = _ref.targetRect,
      entered = _ref.entered,
      prevDimensions = _ref.prevDimensions;

  if (!positionerRect) {
    return {
      positionerStyle: {
        visibility: 'hidden'
      }
    };
  }

  var height;
  var width;

  if (entered || !positionerRef.current) {
    height = Math.round(positionerRect.height);
    width = Math.round(positionerRect.width);
  } else {
    var prevHeight = prevDimensions.height,
        prevWidth = prevDimensions.width;
    height = Math.max(positionerRef.current.offsetHeight, prevHeight);
    width = Math.max(positionerRef.current.offsetWidth, prevWidth);
  }

  var viewportHeight = document.documentElement.clientHeight;
  var viewportWidth = document.documentElement.clientWidth;

  var _getPosition = getPosition({
    position: position,
    targetRect: targetRect,
    targetOffset: targetOffset,
    dimensions: {
      height: height,
      width: width
    },
    viewport: {
      width: viewportWidth,
      height: viewportHeight
    },
    viewportOffset: bodyOffset
  }),
      rect = _getPosition.rect,
      transformOrigin = _getPosition.transformOrigin;

  return {
    positionerStyle: {
      transform: "translate3d(".concat(rect.left, "px, ").concat(rect.top, "px, 0px)")
    },
    transformOrigin: transformOrigin
  };
};

var PositionerImpl = function PositionerImpl(_ref2) {
  var style = _ref2.style,
      position = _ref2.position,
      positioner = _ref2.positioner,
      targetRef = _ref2.targetRef,
      positionerRefFromProps = _ref2.positionerRef;
  var prevDimensions = useRef({
    height: 0,
    width: 0
  });
  var positionerRefFromSelf = useRef(null);
  var positionerRef = useMemo(function () {
    return positionerRefFromProps || positionerRefFromSelf;
  }, [positionerRefFromProps]);
  var targetRect = useRect(targetRef);
  var positionerRect = useRect(positionerRef, {
    onChange: function onChange(rect) {
      prevDimensions.current = {
        height: rect.height,
        width: rect.width
      };
    }
  });

  if (!targetRect) {
    return null;
  }

  var _getPositionerProps = getPositionerProps({
    entered: style.opacity.get() === 1,
    position: position,
    positionerRef: positionerRef,
    positionerRect: positionerRect,
    targetRect: targetRect,
    prevDimensions: prevDimensions.current
  }),
      positionerStyle = _getPositionerProps.positionerStyle,
      transformOrigin = _getPositionerProps.transformOrigin;

  return /*#__PURE__*/React.createElement(StyledPositionerWrapper, {
    style: positionerStyle
  }, /*#__PURE__*/React.createElement(animated.div, {
    ref: positionerRef,
    style: _objectSpread(_objectSpread({}, style), {}, {
      transformOrigin: transformOrigin
    })
  }, positioner instanceof Function ? positioner({
    ref: positionerRef
  }) : positioner));
};

PositionerImpl.propTypes = {
  style: _pt.shape({}).isRequired
};
export default PositionerImpl;