import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _typeof from "@babel/runtime/helpers/typeof";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["id", "width", "size", "creatable", "clearable", "disabled", "loading", "multiple", "searchable", "options", "value", "defaultValue", "placeholder", "menu", "itemSize", "optionsMaxHeight", "onChange", "noOptionsMessage", "formatCreateLabel", "isValidNewOption", "onCreateOption", "onVisibleChange", "onBlur"];
import Downshift from 'downshift3';
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { Flex } from '../Layout';
import { Popover } from '../Popover';
import { Position } from '../constants';
import { useFormField } from '../FormField';
import ClearIcon from './ClearIcon';
import MultiDownshift from './MultiDownshift';
import SelectArrow from './SelectArrow';
import SelectInput from './SelectInput';
import SelectOptions from './SelectOptions';
import SelectedOption from './SelectedOption';
import { Loading, SelectWrapper, StyledSelect } from './styles';
import { getDataTestId, itemToString } from './utils';

var Select = function Select(_ref) {
  var id = _ref.id,
      _ref$width = _ref.width,
      width = _ref$width === void 0 ? 240 : _ref$width,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? 'md' : _ref$size,
      _ref$creatable = _ref.creatable,
      creatable = _ref$creatable === void 0 ? false : _ref$creatable,
      _ref$clearable = _ref.clearable,
      clearable = _ref$clearable === void 0 ? false : _ref$clearable,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      _ref$loading = _ref.loading,
      loading = _ref$loading === void 0 ? false : _ref$loading,
      _ref$multiple = _ref.multiple,
      multiple = _ref$multiple === void 0 ? false : _ref$multiple,
      _ref$searchable = _ref.searchable,
      searchable = _ref$searchable === void 0 ? false : _ref$searchable,
      options = _ref.options,
      value = _ref.value,
      defaultValue = _ref.defaultValue,
      _ref$placeholder = _ref.placeholder,
      placeholder = _ref$placeholder === void 0 ? '' : _ref$placeholder,
      menu = _ref.menu,
      _ref$itemSize = _ref.itemSize,
      itemSize = _ref$itemSize === void 0 ? 36 : _ref$itemSize,
      _ref$optionsMaxHeight = _ref.optionsMaxHeight,
      optionsMaxHeight = _ref$optionsMaxHeight === void 0 ? 180 : _ref$optionsMaxHeight,
      onChange = _ref.onChange,
      noOptionsMessage = _ref.noOptionsMessage,
      formatCreateLabel = _ref.formatCreateLabel,
      isValidNewOption = _ref.isValidNewOption,
      onCreateOption = _ref.onCreateOption,
      onVisibleChange = _ref.onVisibleChange,
      onBlur = _ref.onBlur,
      props = _objectWithoutProperties(_ref, _excluded);

  var _useFormField = useFormField({
    id: id,
    value: value,
    defaultValue: defaultValue
  }),
      _useFormField2 = _slicedToArray(_useFormField, 3),
      invalid = _useFormField2[0],
      labelId = _useFormField2[1],
      setValue = _useFormField2[2];

  var wrapperRef = useRef(null);
  var inputRef = useRef(null);

  var _useState = useState(0),
      _useState2 = _slicedToArray(_useState, 2),
      selectWidth = _useState2[0],
      setSelectWidth = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      visible = _useState4[0],
      setVisible = _useState4[1];

  var _useState5 = useState(''),
      _useState6 = _slicedToArray(_useState5, 2),
      inputValue = _useState6[0],
      setInputValue = _useState6[1];

  useEffect(function () {
    if (visible && wrapperRef.current) {
      setSelectWidth(wrapperRef.current.offsetWidth);
    }

    if (onVisibleChange) {
      onVisibleChange(visible);
    }
  }, [visible, onVisibleChange]);
  var handleChange = useCallback(function (selection) {
    if (inputRef.current && !multiple) {
      inputRef.current.blur();
    }

    var isCreate = selection && selection.label === 'CREATE_OPTION';

    if (isCreate && onCreateOption) {
      onCreateOption(selection.value);
    }

    if (!isCreate && onChange) {
      onChange(selection);
      setValue(selection);
    }

    if (!multiple || isCreate) {
      setVisible(false);
    } else {
      setInputValue('');
    }
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [multiple, onChange, onCreateOption, setValue]);
  var getHighlightedIndex = useCallback(function (item) {
    if (multiple) {
      return null;
    }

    var index = options.findIndex(function (option) {
      return itemToString(option) === itemToString(item);
    });

    if (index === null) {
      return null;
    }

    return index;
  }, [multiple, options]);
  var RenderComponent = useMemo(function () {
    return multiple ? MultiDownshift : Downshift;
  }, [multiple]);
  return /*#__PURE__*/React.createElement(RenderComponent, {
    id: labelId,
    selectedItem: value,
    isOpen: visible,
    initialSelectedItem: defaultValue,
    defaultHighlightedIndex: getHighlightedIndex(value || defaultValue),
    stateReducer: function stateReducer(state, changes) {
      // if the user is opening the menu, then let's make sure
      // that the highlighted index is set to the selected index
      if (_typeof(changes) === 'object' && changes.type === Downshift.stateChangeTypes.clickItem) {
        return _objectSpread(_objectSpread({}, changes), {}, {
          highlightedIndex: getHighlightedIndex(changes.selectedItem)
        });
      }

      return changes;
    },
    itemToString: itemToString,
    inputValue: inputValue,
    onInputValueChange: function onInputValueChange(newInputValue) {
      if (!multiple && newInputValue !== 'CREATE_OPTION') {
        setInputValue(newInputValue);
      }
    },
    onChange: handleChange,
    scrollIntoView: function scrollIntoView() {}
  }, function (_ref2) {
    var downshiftId = _ref2.id,
        getRootProps = _ref2.getRootProps,
        getMenuProps = _ref2.getMenuProps,
        getItemProps = _ref2.getItemProps,
        getInputProps = _ref2.getInputProps,
        highlightedIndex = _ref2.highlightedIndex,
        selectedItem = _ref2.selectedItem,
        clearSelection = _ref2.clearSelection,
        _ref2$selectedItems = _ref2.selectedItems,
        selectedItems = _ref2$selectedItems === void 0 ? [] : _ref2$selectedItems,
        getRemoveButtonProps = _ref2.getRemoveButtonProps,
        removeItem = _ref2.removeItem;
    return /*#__PURE__*/React.createElement(SelectWrapper, _objectSpread({}, getRootProps({
      ref: wrapperRef,
      style: {
        width: width
      }
    })), /*#__PURE__*/React.createElement(Popover, {
      visible: visible,
      position: Position.BOTTOM_LEFT,
      minWidth: selectWidth,
      onVisibleChange: function onVisibleChange(newVisible) {
        if (disabled || loading) {
          return;
        }

        if (!newVisible && inputRef.current) {
          inputRef.current.blur();
        }

        setVisible(newVisible);
      },
      p: "0",
      content: /*#__PURE__*/React.createElement(SelectOptions, {
        visible: visible,
        creatable: creatable,
        searchable: searchable,
        multiple: multiple,
        itemSize: itemSize,
        optionsMaxHeight: optionsMaxHeight,
        getMenuProps: getMenuProps,
        getItemProps: getItemProps,
        options: options,
        inputValue: inputValue,
        menu: menu,
        highlightedIndex: highlightedIndex,
        selectedItem: selectedItem,
        selectedItems: selectedItems,
        noOptionsMessage: noOptionsMessage,
        formatCreateLabel: formatCreateLabel,
        isValidNewOption: isValidNewOption,
        "data-testid": props['data-testid']
      })
    }, /*#__PURE__*/React.createElement(StyledSelect, _objectSpread(_objectSpread({
      id: downshiftId
    }, getDataTestId(props)), {}, {
      invalid: invalid,
      size: size,
      focused: visible,
      disabled: disabled || loading,
      multiple: multiple,
      onClick: function onClick() {
        if (!visible) {
          if (searchable || creatable || multiple) {
            setInputValue('');
          }

          if (inputRef.current) {
            inputRef.current.focus();
          }
        }
      }
    }), /*#__PURE__*/React.createElement(Flex, {
      flex: "auto",
      flexWrap: "wrap",
      alignItems: "center",
      overflow: "hidden"
    }, multiple && selectedItems.map(function (item, index) {
      return /*#__PURE__*/React.createElement(SelectedOption, {
        key: itemToString(item),
        index: index,
        size: size,
        "data-testid": props['data-testid'],
        removeButtonProps: getRemoveButtonProps({
          item: item,
          onClick: function onClick() {
            if (inputRef.current) {
              inputRef.current.focus();
            }
          }
        })
      }, itemToString(item));
    }), /*#__PURE__*/React.createElement(SelectInput, {
      ref: inputRef,
      visible: visible,
      searchable: searchable,
      creatable: creatable,
      multiple: multiple,
      inputValue: inputValue,
      placeholder: placeholder,
      options: options,
      selectedItem: selectedItem,
      selectedItems: selectedItems,
      removeItem: removeItem,
      onChange: function onChange(event) {
        return setInputValue(event.currentTarget.value);
      },
      onBlur: onBlur,
      getInputProps: getInputProps,
      "data-testid": props['data-testid']
    })), clearable && selectedItem && /*#__PURE__*/React.createElement(ClearIcon, {
      clearSelection: clearSelection,
      "data-testid": props['data-testid']
    }), loading ? /*#__PURE__*/React.createElement(Loading, {
      title: "loading"
    }) : /*#__PURE__*/React.createElement(SelectArrow, {
      on: visible
    }))));
  });
};

export { Select };