import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _typeof from "@babel/runtime/helpers/typeof";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _pt from "prop-types";
var _excluded = ["visible", "creatable", "searchable", "multiple", "itemSize", "optionsMaxHeight", "getMenuProps", "getItemProps", "options", "inputValue", "menu", "highlightedIndex", "selectedItem", "selectedItems", "noOptionsMessage", "formatCreateLabel", "isValidNewOption"];
import React, { useEffect, useMemo, useRef, useState } from 'react';
import { FixedSizeList } from 'react-window';
import { MdCheck, MdHighlightOff } from 'react-icons/md';
import { Box, Flex } from '../Layout';
import { Heading } from '../Typography';
import { Icon } from '../Icon';
import { useLocale } from '../locale';
import { StyledSelectOption } from './styles';
import { fuzzyFilter, getDataTestId, itemToString } from './utils';

var DefaultNoOptionsMessage = function DefaultNoOptionsMessage() {
  var _useLocale = useLocale(),
      locale = _useLocale.locale;

  return /*#__PURE__*/React.createElement(Flex, {
    py: "3",
    alignItems: "center",
    justifyContent: "center",
    style: {
      cursor: 'not-allowed'
    }
  }, /*#__PURE__*/React.createElement(Icon, {
    type: MdHighlightOff,
    mr: "2"
  }), /*#__PURE__*/React.createElement(Heading.H5, {
    color: "gray500"
  }, locale.Select.noDataText));
};

var SelectOptions = function SelectOptions(_ref) {
  var visible = _ref.visible,
      creatable = _ref.creatable,
      searchable = _ref.searchable,
      multiple = _ref.multiple,
      itemSize = _ref.itemSize,
      optionsMaxHeight = _ref.optionsMaxHeight,
      getMenuProps = _ref.getMenuProps,
      getItemProps = _ref.getItemProps,
      options = _ref.options,
      inputValue = _ref.inputValue,
      menu = _ref.menu,
      highlightedIndex = _ref.highlightedIndex,
      selectedItem = _ref.selectedItem,
      selectedItems = _ref.selectedItems,
      _ref$noOptionsMessage = _ref.noOptionsMessage,
      noOptionsMessage = _ref$noOptionsMessage === void 0 ? function () {
    return /*#__PURE__*/React.createElement(DefaultNoOptionsMessage, null);
  } : _ref$noOptionsMessage,
      _ref$formatCreateLabe = _ref.formatCreateLabel,
      formatCreateLabel = _ref$formatCreateLabe === void 0 ? function (_ref2) {
    var value = _ref2.value;
    return "Create new option: ".concat(value);
  } : _ref$formatCreateLabe,
      _ref$isValidNewOption = _ref.isValidNewOption,
      isValidNewOption = _ref$isValidNewOption === void 0 ? function (value) {
    return value.trim() !== '';
  } : _ref$isValidNewOption,
      props = _objectWithoutProperties(_ref, _excluded);

  var listRef = useRef(null);

  var _useState = useState(inputValue),
      _useState2 = _slicedToArray(_useState, 2),
      prevSearchValue = _useState2[0],
      setPrevSearchValue = _useState2[1];

  var _useState3 = useState(undefined),
      _useState4 = _slicedToArray(_useState3, 2),
      createOptionWidth = _useState4[0],
      setCreateOptionWidth = _useState4[1];

  var createOptionRef = useRef(null);
  useEffect(function () {
    if (listRef.current && highlightedIndex) {
      listRef.current.scrollToItem(highlightedIndex);
    }
  }, [highlightedIndex]);
  useEffect(function () {
    // save previous inputValue to prevent options reset when options animating fade out
    if (visible) {
      setPrevSearchValue(inputValue);
    } else {
      setCreateOptionWidth(undefined);
    }

    if (createOptionRef.current) {
      setCreateOptionWidth(createOptionRef.current.scrollWidth + 10);
    }
  }, [inputValue, visible]);
  var filteredItems = useMemo(function () {
    // filter options when searchable or creatable or multiple
    if (searchable || creatable || multiple) {
      var filter = fuzzyFilter(itemToString);
      return prevSearchValue.trim() === '' ? options : filter(options, prevSearchValue);
    }

    return options;
  }, [creatable, multiple, options, prevSearchValue, searchable]);
  var additionItems = useMemo(function () {
    // if creatable is true, append create option to options
    if (creatable && isValidNewOption(visible ? inputValue : prevSearchValue)) {
      return [{
        label: 'CREATE_OPTION',
        value: visible ? inputValue : prevSearchValue
      }];
    }

    return [];
  }, [creatable, inputValue, isValidNewOption, prevSearchValue, visible]);
  var items = useMemo(function () {
    return [].concat(_toConsumableArray(filteredItems), _toConsumableArray(additionItems));
  }, [additionItems, filteredItems]);

  if (items.length === 0) {
    return /*#__PURE__*/React.createElement(React.Fragment, null, noOptionsMessage());
  }

  return /*#__PURE__*/React.createElement(Box, _objectSpread(_objectSpread({
    borderRadius: "lg",
    overflow: "hidden",
    style: {
      minWidth: createOptionWidth
    }
  }, getDataTestId(props, 'menu')), getMenuProps()), /*#__PURE__*/React.createElement(FixedSizeList, {
    ref: listRef,
    width: "100%",
    height: Math.min(items.length * itemSize, optionsMaxHeight),
    itemSize: itemSize,
    itemCount: items.length,
    initialScrollOffset: highlightedIndex ? highlightedIndex * itemSize : 0
  }, function (_ref3) {
    var index = _ref3.index,
        style = _ref3.style;
    var option = items[index];
    var isCreateOption = option.label === 'CREATE_OPTION';
    var optionString = itemToString(option);
    var hovered = visible && highlightedIndex === index;
    var disabled = _typeof(option) === 'object' && 'disabled' in option ? option.disabled : undefined;
    var active = multiple ? selectedItems.map(itemToString).includes(optionString) : itemToString(selectedItem) === optionString;
    var content = isCreateOption ? formatCreateLabel({
      value: option.value,
      active: active,
      hovered: hovered
    }) : optionString;
    return /*#__PURE__*/React.createElement(StyledSelectOption, _objectSpread(_objectSpread({
      key: optionString,
      active: active,
      hovered: hovered,
      disabled: disabled,
      style: style
    }, getDataTestId(props, "item-".concat(index))), getItemProps({
      index: index,
      item: option,
      disabled: disabled
    })), /*#__PURE__*/React.createElement("div", {
      ref: isCreateOption ? createOptionRef : undefined,
      title: isCreateOption ? undefined : String(content)
    }, multiple && (active ? /*#__PURE__*/React.createElement(Icon, {
      type: MdCheck,
      fill: hovered ? 'light' : 'primaryLight',
      size: "16",
      mr: "8px"
    }) : /*#__PURE__*/React.createElement(Box, {
      width: "16px",
      mr: "8px"
    })), content));
  }), menu);
};

SelectOptions.propTypes = {
  visible: _pt.bool.isRequired,
  creatable: _pt.bool.isRequired,
  searchable: _pt.bool.isRequired,
  multiple: _pt.bool.isRequired,
  itemSize: _pt.number.isRequired,
  optionsMaxHeight: _pt.number.isRequired,
  getMenuProps: _pt.func.isRequired,
  getItemProps: _pt.func.isRequired,
  options: _pt.arrayOf(_pt.oneOfType([_pt.shape({
    label: _pt.node.isRequired,
    value: _pt.oneOfType([_pt.string, _pt.number]).isRequired,
    disabled: _pt.bool
  }), _pt.string, _pt.number])).isRequired,
  inputValue: _pt.string.isRequired,
  menu: _pt.node,
  highlightedIndex: _pt.oneOfType([_pt.number, _pt.oneOf([null])]),
  selectedItem: _pt.oneOfType([_pt.shape({
    label: _pt.node.isRequired,
    value: _pt.oneOfType([_pt.string, _pt.number]).isRequired,
    disabled: _pt.bool
  }), _pt.string, _pt.number]),
  selectedItems: _pt.arrayOf(_pt.oneOfType([_pt.shape({
    label: _pt.node.isRequired,
    value: _pt.oneOfType([_pt.string, _pt.number]).isRequired,
    disabled: _pt.bool
  }), _pt.string, _pt.number])),
  noOptionsMessage: _pt.func,
  formatCreateLabel: _pt.func,
  isValidNewOption: _pt.func,
  'data-testid': _pt.string
};
export default SelectOptions;