import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _minBy from "ramda/src/minBy";
import _clamp from "ramda/src/clamp";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _pt from "prop-types";
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import SliderDot from './SliderDot';
import { getPageX, updateRangeValues } from './utils';
import { StyledSlider, StyledSliderActiveBar, StyledSliderBar, StyledSliderContainer } from './styles';

var Slider = function Slider(_ref) {
  var _ref$min = _ref.min,
      min = _ref$min === void 0 ? 0 : _ref$min,
      _ref$max = _ref.max,
      max = _ref$max === void 0 ? 100 : _ref$max,
      _ref$step = _ref.step,
      step = _ref$step === void 0 ? 1 : _ref$step,
      _ref$range = _ref.range,
      range = _ref$range === void 0 ? false : _ref$range,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      valueFromProps = _ref.value,
      initialDefaultValue = _ref.defaultValue,
      onChange = _ref.onChange;
  var sliderRef = useRef(null);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      active = _useState2[0],
      setActive = _useState2[1];

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      changedValue = _useState4[0],
      setChangedValue = _useState4[1];

  var _useState5 = useState(initialDefaultValue || (range ? [0, 0] : 0)),
      _useState6 = _slicedToArray(_useState5, 2),
      defaultValue = _useState6[0],
      setDefaultValue = _useState6[1];

  var _useState7 = useState(0),
      _useState8 = _slicedToArray(_useState7, 2),
      stepOffset = _useState8[0],
      setStepOffset = _useState8[1];

  var _useState9 = useState(null),
      _useState10 = _slicedToArray(_useState9, 2),
      dragStart = _useState10[0],
      setDragStart = _useState10[1];

  var value = valueFromProps || defaultValue;
  var denominator = max - min;
  var handleChange = useCallback(function (newValue) {
    if (onChange) {
      onChange(newValue);
    } else {
      setDefaultValue(newValue);
    }
  }, [onChange]);
  useEffect(function () {
    if (sliderRef.current) {
      var offset = sliderRef.current.offsetWidth / denominator * step;
      setStepOffset(offset);
    }
  }, [denominator, step]);
  useEffect(function () {
    var handleMoveStop = function handleMoveStop() {
      return setDragStart(null);
    };

    var handleMoving = function handleMoving(event) {
      if (dragStart) {
        var pageX = getPageX({
          event: event
        });
        var offsetX = pageX - dragStart.x;
        var offsetValue = Math.round(offsetX / stepOffset) * step;

        var newValue = _clamp(min, max, dragStart.value + offsetValue);

        var changeVal = range ? updateRangeValues({
          value: value,
          replaceValue: dragStart.value,
          newValue: newValue
        }) : newValue;

        if (range) {
          setChangedValue(newValue);
        }

        handleChange(changeVal);
      }
    };

    if (dragStart) {
      var moveListenerName = dragStart.touch ? 'touchmove' : 'mousemove';
      var endListenerName = dragStart.touch ? 'touchend' : 'mouseup';
      document.addEventListener(moveListenerName, handleMoving);
      document.addEventListener(endListenerName, handleMoveStop);
    }

    return function () {
      if (dragStart) {
        var _moveListenerName = dragStart.touch ? 'touchmove' : 'mousemove';

        var _endListenerName = dragStart.touch ? 'touchend' : 'mouseup';

        document.removeEventListener(_moveListenerName, handleMoving);
        document.removeEventListener(_endListenerName, handleMoveStop);
        setChangedValue(undefined);
        setActive(false);
      }
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [dragStart]);
  var valuePercentage = useMemo(function () {
    return "".concat((range ? value[1] - value[0] : value - min) / denominator * 100, "%");
  }, [denominator, min, range, value]);
  var handleSlideStart = useCallback(function (event) {
    if (disabled) {
      return;
    }

    var isTouchEvent = ('touches' in event);

    if (isTouchEvent) {
      event.stopPropagation();
    }

    var _event$currentTarget$ = event.currentTarget.getBoundingClientRect(),
        left = _event$currentTarget$.left;

    var movedX = getPageX({
      event: event
    });
    var offsetX = movedX - left;
    var offsetValue = Math.round(offsetX / stepOffset) * step + min;

    var newValue = _clamp(min, max, offsetValue);

    if (range) {
      var replaceValue = _minBy.apply(void 0, [function (v) {
        return Math.abs(newValue - v);
      }].concat(_toConsumableArray(value)));

      var updatedValue = updateRangeValues({
        value: value,
        replaceValue: replaceValue,
        newValue: newValue
      });
      setChangedValue(newValue);
      handleChange(updatedValue);
    } else {
      handleChange(newValue);
    }

    setActive(true);
    setDragStart({
      x: movedX,
      value: newValue,
      touch: isTouchEvent
    });
  }, [disabled, handleChange, max, min, range, step, stepOffset, value]);
  return /*#__PURE__*/React.createElement(StyledSliderContainer, {
    min: min,
    max: max
  }, /*#__PURE__*/React.createElement(StyledSlider, {
    ref: sliderRef,
    disabled: disabled,
    onMouseDown: handleSlideStart,
    onTouchStart: handleSlideStart,
    onTouchEnd: function onTouchEnd(e) {
      return e.preventDefault();
    }
  }, /*#__PURE__*/React.createElement(StyledSliderBar, null), /*#__PURE__*/React.createElement(StyledSliderActiveBar, {
    style: {
      left: range ? "".concat((value[0] - min) / denominator * 100, "%") : 0,
      width: valuePercentage
    }
  }), /*#__PURE__*/React.createElement(SliderDot, {
    range: range,
    value: value,
    active: active,
    changedValue: changedValue,
    denominator: denominator,
    min: min,
    valuePercentage: valuePercentage
  })));
};

Slider.propTypes = {
  min: _pt.number,
  max: _pt.number,
  step: _pt.number,
  range: _pt.bool,
  disabled: _pt.bool,
  onChange: _pt.func
};
export { Slider };