import _pt from "prop-types";
import React, { useContext } from 'react';
import { MdClose, MdDone } from 'react-icons/md';
import { Flex } from '../Layout';
import { Icon } from '../Icon';
import StepContext from './StepContext';
import { Description, StepsIconItem, Title } from './styles';

var StepIcon = function StepIcon(_ref) {
  var status = _ref.status,
      count = _ref.count,
      isLast = _ref.isLast,
      direction = _ref.direction,
      tailColor = _ref.tailColor,
      onClick = _ref.onClick;
  var isFinish = status === 'finish';
  var renderIcon = ['finish', 'error'].includes(status);
  return /*#__PURE__*/React.createElement(StepsIconItem, {
    status: status,
    isLast: isLast,
    direction: direction,
    tailColor: tailColor,
    onClick: onClick
  }, renderIcon ? /*#__PURE__*/React.createElement(Icon, {
    size: "20",
    type: isFinish ? MdDone : MdClose,
    fill: isFinish ? 'primary' : 'error',
    style: {
      pointerEvents: 'none'
    }
  }) : count + 1);
};

StepIcon.propTypes = {
  count: _pt.number.isRequired,
  onClick: _pt.func.isRequired
};

var Step = function Step(_ref2) {
  var title = _ref2.title,
      _ref2$description = _ref2.description,
      description = _ref2$description === void 0 ? null : _ref2$description;

  var _useContext = useContext(StepContext),
      count = _useContext.count,
      status = _useContext.status,
      isLast = _useContext.isLast,
      tailColor = _useContext.tailColor,
      direction = _useContext.direction,
      onCurrentChange = _useContext.onCurrentChange;

  return /*#__PURE__*/React.createElement(Flex, {
    flex: isLast ? 'none' : 'auto',
    mr: "16px",
    mb: direction === 'vertical' ? '6px' : 0,
    overflow: "hidden",
    minHeight: direction === 'vertical' ? '64px' : 'auto'
  }, /*#__PURE__*/React.createElement(StepIcon, {
    status: status,
    count: count,
    isLast: isLast,
    direction: direction,
    tailColor: tailColor,
    onClick: function onClick() {
      if (onCurrentChange) {
        onCurrentChange(count);
      }
    }
  }), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(Title, {
    status: status,
    direction: direction,
    isLast: isLast,
    tailColor: tailColor
  }, title), description && /*#__PURE__*/React.createElement(Description, {
    status: status,
    direction: direction
  }, description)));
};

Step.propTypes = {
  title: _pt.string.isRequired,
  description: _pt.string
};
export default Step;