import React, { Children, isValidElement } from 'react';
import { Flex } from '../Layout';
import Step from './Step';
import StepContext from './StepContext';

var getStatus = function getStatus(_ref) {
  var current = _ref.current,
      count = _ref.count;

  if (current === count) {
    return 'progress';
  }

  if (current > count) {
    return 'finish';
  }

  if (current < count) {
    return 'wait';
  }

  return 'progress';
};

var getTailColor = function getTailColor(_ref2) {
  var count = _ref2.count,
      status = _ref2.status,
      steps = _ref2.steps;
  var nextStep = steps[count + 1];

  if ( /*#__PURE__*/isValidElement(nextStep) && nextStep.props.status === 'error') {
    return 'error';
  }

  if (status === 'progress' || status === 'wait') {
    return 'gray';
  }

  return 'primary';
};

var Steps = function Steps(_ref3) {
  var children = _ref3.children,
      current = _ref3.current,
      onCurrentChange = _ref3.onCurrentChange,
      _ref3$direction = _ref3.direction,
      direction = _ref3$direction === void 0 ? 'horizontal' : _ref3$direction;
  var totalCount = Children.count(children) - 1;
  var steps = Children.toArray(children);
  return /*#__PURE__*/React.createElement(Flex, {
    flexDirection: direction === 'horizontal' ? 'row' : 'column'
  }, steps.map(function (step, count) {
    if ( /*#__PURE__*/isValidElement(step)) {
      var status = step.props.status || getStatus({
        current: current,
        count: count
      });
      var isLast = count === totalCount;
      var tailColor = getTailColor({
        count: count,
        status: status,
        steps: steps
      });
      return /*#__PURE__*/React.createElement(StepContext.Provider, {
        key: step.key || count,
        value: {
          count: count,
          status: status,
          isLast: isLast,
          tailColor: tailColor,
          direction: direction,
          onCurrentChange: onCurrentChange
        }
      }, step);
    }

    return null;
  }));
};

Steps.Step = Step;
export { Steps };