import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { createContext, memo, useCallback, useContext, useEffect, useMemo, useState } from 'react';
var FixedColumnContext = /*#__PURE__*/createContext({
  setColumn: function setColumn() {},
  getColumnFixedInfo: function getColumnFixedInfo() {
    return {};
  }
});

var useFixedColumnContext = function useFixedColumnContext() {
  return useContext(FixedColumnContext);
};

var useFixedHeadColumn = function useFixedHeadColumn(_ref) {
  var index = _ref.index,
      width = _ref.width,
      fixed = _ref.fixed,
      style = _ref.style,
      colSpan = _ref.colSpan;

  var _useFixedColumnContex = useFixedColumnContext(),
      setColumn = _useFixedColumnContex.setColumn,
      getColumnFixedInfo = _useFixedColumnContex.getColumnFixedInfo;

  useEffect(function () {
    setColumn({
      index: index,
      width: width,
      fixed: fixed,
      colSpan: colSpan
    });
  }, [index, width, fixed, colSpan, setColumn]);
  return useMemo(function () {
    return getColumnFixedInfo({
      index: index,
      style: style
    });
  }, [getColumnFixedInfo, index, style]);
};

var FixedColumnContextProvider = /*#__PURE__*/memo(function FixedColumnContextProvider(_ref2) {
  var children = _ref2.children,
      scrollShadowStart = _ref2.scrollShadowStart,
      scrollShadowEnd = _ref2.scrollShadowEnd;

  var _useState = useState([]),
      _useState2 = _slicedToArray(_useState, 2),
      fixedColumns = _useState2[0],
      setFixedColumns = _useState2[1];

  var setColumn = useCallback(function (column) {
    if (column.index === -1) {
      return;
    }

    setFixedColumns(function (prevColumns) {
      return [].concat(_toConsumableArray(prevColumns.slice(0, column.index)), [column], _toConsumableArray(prevColumns.slice(column.index + 1)));
    });
  }, []);
  var fixedColumnsInfo = useMemo(function () {
    return fixedColumns.map(function (fixedColumn, index) {
      if (fixedColumn.fixed === 'left') {
        var offsetWidth = fixedColumns.slice(0, index).reduce(function (prev, curr) {
          return prev + curr.width;
        }, 0);
        var isLastFixedLeft = Math.abs(fixedColumns.slice().reverse().findIndex(function (column) {
          return column.fixed === 'left';
        }) - (fixedColumns.length - 1)) === index;
        return _objectSpread(_objectSpread({}, fixedColumn), {}, {
          isLastFixedLeft: isLastFixedLeft,
          style: {
            left: offsetWidth
          }
        });
      }

      if (fixedColumn.fixed === 'right') {
        var _offsetWidth = fixedColumns.slice(index + 1).reduce(function (prev, curr) {
          return prev + curr.width;
        }, 0);

        var isFirstFixedRight = fixedColumns.findIndex(function (column) {
          return column.fixed === 'right';
        }) === index;
        return _objectSpread(_objectSpread({}, fixedColumn), {}, {
          isFirstFixedRight: isFirstFixedRight,
          style: {
            right: _offsetWidth
          }
        });
      }

      return _objectSpread(_objectSpread({}, fixedColumn), {}, {
        style: {}
      });
    });
  }, [fixedColumns]);
  var getColumnFixedInfo = useCallback(function (_ref3) {
    var index = _ref3.index,
        style = _ref3.style;
    var targetColumn = fixedColumnsInfo[index];

    if (!targetColumn) {
      return {
        style: style
      };
    }

    if ('isLastFixedLeft' in targetColumn && targetColumn.isLastFixedLeft) {
      return {
        fixed: true,
        isLastFixedLeft: targetColumn.isLastFixedLeft,
        showScrollShadowStart: scrollShadowStart,
        style: _objectSpread(_objectSpread({}, style), targetColumn.style)
      };
    }

    if ('isFirstFixedRight' in targetColumn && targetColumn.isFirstFixedRight) {
      return {
        fixed: true,
        isFirstFixedRight: targetColumn.isFirstFixedRight,
        showScrollShadowEnd: scrollShadowEnd,
        style: _objectSpread(_objectSpread({}, style), targetColumn.style)
      };
    }

    return {
      fixed: typeof targetColumn.fixed === 'string',
      style: _objectSpread(_objectSpread({}, style), targetColumn.style)
    };
  }, [fixedColumnsInfo, scrollShadowStart, scrollShadowEnd]);
  var value = useMemo(function () {
    return {
      setColumn: setColumn,
      getColumnFixedInfo: getColumnFixedInfo
    };
  }, [setColumn, getColumnFixedInfo]);
  return /*#__PURE__*/React.createElement(FixedColumnContext.Provider, {
    value: value
  }, children);
});
export { FixedColumnContextProvider, useFixedColumnContext, useFixedHeadColumn };