import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["header", "footer", "width", "maxHeight", "textAlign", "children"];
import React, { forwardRef, useCallback, useEffect, useRef, useState } from 'react';
import { useComposedRefs } from '@reach/utils/compose-refs';
import { useMeasure } from '../hooks';
import Column from './Column';
import HeadColumn from './HeadColumn';
import { FixedColumnContextProvider } from './FixedColumnContext';
import { Row, StyledTable, StyledTableWrapper, TableWrapper } from './styles';
var Head = /*#__PURE__*/forwardRef(function Head(_ref, ref) {
  var children = _ref.children;
  return /*#__PURE__*/React.createElement("thead", {
    ref: ref
  }, /*#__PURE__*/React.createElement(Row, null, children));
});
var Body = /*#__PURE__*/forwardRef(function Body(_ref2, ref) {
  var children = _ref2.children;
  return /*#__PURE__*/React.createElement("tbody", {
    ref: ref
  }, children);
});
var Table = /*#__PURE__*/forwardRef(function Table(_ref3, forwardedRef) {
  var header = _ref3.header,
      footer = _ref3.footer,
      _ref3$width = _ref3.width,
      width = _ref3$width === void 0 ? '100%' : _ref3$width,
      maxHeight = _ref3.maxHeight,
      _ref3$textAlign = _ref3.textAlign,
      textAlign = _ref3$textAlign === void 0 ? 'center' : _ref3$textAlign,
      children = _ref3.children,
      props = _objectWithoutProperties(_ref3, _excluded);

  var ownRef = useRef(null);
  var forkedRef = useComposedRefs(forwardedRef, ownRef);

  var _useMeasure = useMeasure(forkedRef),
      _useMeasure2 = _slicedToArray(_useMeasure, 2),
      ref = _useMeasure2[0].ref,
      bounds = _useMeasure2[1];

  var _useState = useState({
    start: false,
    end: false
  }),
      _useState2 = _slicedToArray(_useState, 2),
      scrollShadow = _useState2[0],
      setScrollShadow = _useState2[1];

  var handleUpdateScrollShadow = useCallback(function (target) {
    var scrollLeft = target.scrollLeft,
        scrollWidth = target.scrollWidth,
        offsetWidth = target.offsetWidth;
    setScrollShadow({
      start: scrollLeft > 0,
      end: scrollLeft + offsetWidth < scrollWidth
    });
  }, []);
  useEffect(function () {
    if (ownRef.current) {
      handleUpdateScrollShadow(ownRef.current);
    }
  }, [bounds.offsetWidth, handleUpdateScrollShadow]);
  var optionsProps = {
    hasHeader: Boolean(header),
    hasFooter: Boolean(footer)
  };
  var table = /*#__PURE__*/React.createElement(FixedColumnContextProvider, {
    scrollShadowStart: scrollShadow.start,
    scrollShadowEnd: scrollShadow.end
  }, /*#__PURE__*/React.createElement(StyledTableWrapper, _objectSpread(_objectSpread({
    ref: ref,
    width: width,
    maxHeight: maxHeight,
    textAlign: textAlign,
    onScroll: function onScroll(event) {
      return handleUpdateScrollShadow(event.currentTarget);
    }
  }, optionsProps), props), /*#__PURE__*/React.createElement(StyledTable, null, children)));

  if (header || footer) {
    return /*#__PURE__*/React.createElement(TableWrapper, _objectSpread(_objectSpread({
      ref: ref,
      width: width
    }, optionsProps), props), header, table, footer);
  }

  return table;
});
Table.displayName = 'Table';
Table.Head = Head;
Table.HeadColumn = HeadColumn;
Table.Body = Body;
Table.Row = Row;
Table.Column = Column;
export { Table };