import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _pt from "prop-types";
var _excluded = ["onBeforeSelect", "onSelect", "onClear", "disabled", "texts", "buttonProps"];
import _regeneratorRuntime from "@babel/runtime/regenerator";
import React, { useCallback, useMemo, useReducer } from 'react';
import styled from 'styled-components';
import { useDropzone } from 'react-dropzone';
import { MdCheck, MdClose, MdFileUpload } from 'react-icons/md';
import { Box, Flex } from '../Layout';
import { Button } from '../Button';
import { Icon } from '../Icon';
import { useLocale } from '../locale';
var FileList = styled.div.withConfig({
  componentId: "sc-16fkh04-0"
})(["display:inline-flex;flex:none;flex-direction:column;"]);

var getUploadText = function getUploadText(_ref) {
  var uploading = _ref.uploading,
      uploaded = _ref.uploaded,
      failed = _ref.failed,
      texts = _ref.texts;

  if (uploading) {
    return texts.uploadingText;
  }

  if (failed) {
    return texts.failedText;
  }

  if (uploaded) {
    return texts.uploadedText;
  }

  return texts.uploadText;
};

var getUploadIcon = function getUploadIcon(_ref2) {
  var uploaded = _ref2.uploaded,
      failed = _ref2.failed;

  if (uploaded) {
    return MdCheck;
  }

  if (failed) {
    return MdClose;
  }

  return MdFileUpload;
};

var StyledFileItem = styled(Flex).withConfig({
  componentId: "sc-16fkh04-1"
})(["&:hover{background-color:", ";}"], function (p) {
  return p.theme.colors.gray200;
});

var FileItem = function FileItem(_ref3) {
  var file = _ref3.file,
      uploaded = _ref3.uploaded,
      onClear = _ref3.onClear;
  return /*#__PURE__*/React.createElement(StyledFileItem, {
    alignItems: "center",
    mt: "1",
    p: "1",
    borderRadius: "base"
  }, uploaded && /*#__PURE__*/React.createElement(Icon, {
    type: MdClose,
    cursor: "pointer",
    mr: "1",
    size: "16",
    onClick: function onClick() {
      return onClear(file);
    }
  }), /*#__PURE__*/React.createElement(Box, {
    color: "gray600",
    fontSize: "sm"
  }, file.name));
};

FileItem.propTypes = {
  uploaded: _pt.bool.isRequired,
  onClear: _pt.func.isRequired
};
var START_UPLOADING = 'START_UPLOADING';
var UPLOAD_SUCCESS = 'UPLOAD_SUCCESS';
var UPLOAD_FAILED = 'UPLOAD_FAILED';
var CLEAR_FILES = 'CLEAR_FILES';

var uploadReducer = function uploadReducer(state, action) {
  switch (action.type) {
    case START_UPLOADING:
      return _objectSpread(_objectSpread({}, state), {}, {
        uploading: true,
        uploaded: false,
        failed: false,
        files: action.payload
      });

    case UPLOAD_SUCCESS:
      return _objectSpread(_objectSpread({}, state), {}, {
        uploading: false,
        uploaded: true,
        failed: false
      });

    case UPLOAD_FAILED:
      return _objectSpread(_objectSpread({}, state), {}, {
        uploading: false,
        uploaded: false,
        failed: true
      });

    case CLEAR_FILES:
      return _objectSpread(_objectSpread({}, state), {}, {
        uploaded: false,
        files: state.files.filter(function (file) {
          return file.name !== action.payload;
        })
      });

    default:
      throw new Error();
  }
};

var Upload = function Upload(_ref4) {
  var onBeforeSelect = _ref4.onBeforeSelect,
      onSelect = _ref4.onSelect,
      onClear = _ref4.onClear,
      disabled = _ref4.disabled,
      _ref4$texts = _ref4.texts,
      texts = _ref4$texts === void 0 ? {} : _ref4$texts,
      _ref4$buttonProps = _ref4.buttonProps,
      buttonProps = _ref4$buttonProps === void 0 ? {} : _ref4$buttonProps,
      props = _objectWithoutProperties(_ref4, _excluded);

  var _useLocale = useLocale(),
      locale = _useLocale.locale;

  var _useReducer = useReducer(uploadReducer, {
    uploading: false,
    uploaded: false,
    failed: false,
    files: []
  }),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var handleSelect = useCallback( /*#__PURE__*/function () {
    var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(selectedFiles) {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              dispatch({
                type: 'START_UPLOADING',
                payload: selectedFiles
              });
              _context.prev = 1;
              _context.next = 4;
              return onSelect(selectedFiles);

            case 4:
              dispatch({
                type: 'UPLOAD_SUCCESS'
              });
              _context.next = 10;
              break;

            case 7:
              _context.prev = 7;
              _context.t0 = _context["catch"](1);
              dispatch({
                type: 'UPLOAD_FAILED'
              });

            case 10:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[1, 7]]);
    }));

    return function (_x) {
      return _ref5.apply(this, arguments);
    };
  }(), [onSelect]);
  var handleClear = useCallback(function (clearFile) {
    dispatch({
      type: 'CLEAR_FILES',
      payload: clearFile.name
    });

    if (onClear) {
      onClear(clearFile);
    }
  }, [onClear]);
  var icon = useMemo(function () {
    return getUploadIcon(state);
  }, [state]);
  var text = useMemo(function () {
    return getUploadText(_objectSpread(_objectSpread({}, state), {}, {
      texts: _objectSpread(_objectSpread({}, locale.Upload), texts)
    }));
  }, [locale.Upload, state, texts]);

  var _useDropzone = useDropzone(_objectSpread({
    onDrop: handleSelect
  }, props)),
      getRootProps = _useDropzone.getRootProps,
      getInputProps = _useDropzone.getInputProps,
      open = _useDropzone.open;

  return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", _objectSpread(_objectSpread({
    tabIndex: -1,
    onKeyPress: function onKeyPress() {},
    role: "button"
  }, getRootProps()), {}, {
    style: {
      display: 'inline-flex',
      flex: 'none'
    },
    onClick: function onClick(event) {
      return event.preventDefault();
    }
  }), /*#__PURE__*/React.createElement("input", _objectSpread({}, getInputProps())), /*#__PURE__*/React.createElement(Button, _objectSpread({
    icon: icon,
    loading: state.uploading,
    disabled: disabled,
    onClick: /*#__PURE__*/function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(event) {
        var success;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                event.preventDefault();

                if (!onBeforeSelect) {
                  _context2.next = 14;
                  break;
                }

                _context2.prev = 2;
                _context2.next = 5;
                return onBeforeSelect();

              case 5:
                success = _context2.sent;

                if (success) {
                  _context2.next = 8;
                  break;
                }

                return _context2.abrupt("return");

              case 8:
                _context2.next = 14;
                break;

              case 10:
                _context2.prev = 10;
                _context2.t0 = _context2["catch"](2);
                console.error(_context2.t0);
                return _context2.abrupt("return");

              case 14:
                open();

              case 15:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, null, [[2, 10]]);
      }));

      return function (_x2) {
        return _ref6.apply(this, arguments);
      };
    }()
  }, buttonProps), text)), /*#__PURE__*/React.createElement("br", null), state.files.length > 0 && /*#__PURE__*/React.createElement(FileList, null, state.files.map(function (file) {
    return /*#__PURE__*/React.createElement(FileItem, {
      key: file.name,
      file: file,
      uploaded: state.uploaded,
      onClear: handleClear
    });
  })));
};

Upload.propTypes = {
  disabled: _pt.bool,
  onSelect: _pt.func.isRequired,
  onClear: _pt.func,
  onBeforeSelect: _pt.func,
  texts: _pt.shape({
    uploadText: _pt.string,
    uploadingText: _pt.string,
    uploadedText: _pt.string,
    failedText: _pt.string
  })
};
export { Upload };