import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import debounce from 'lodash.debounce';
import React, { useCallback, useRef, useState } from 'react';

var useHoverTrigger = function useHoverTrigger(_ref) {
  var visibleFromProps = _ref.visible,
      defaultVisible = _ref.defaultVisible,
      onVisibleChange = _ref.onVisibleChange,
      mouseEnterDelay = _ref.mouseEnterDelay,
      mouseLeaveDelay = _ref.mouseLeaveDelay;
  var cancelEnterDebounce = useRef();
  var cancelLeaveDebounce = useRef();

  var _useState = useState(defaultVisible),
      _useState2 = _slicedToArray(_useState, 2),
      visibleFromSelf = _useState2[0],
      setVisibleFromSelf = _useState2[1];

  var hasVisibleFromProps = typeof visibleFromProps !== 'undefined';
  var visible = hasVisibleFromProps ? visibleFromProps : visibleFromSelf;
  var open = useCallback(function () {
    if (onVisibleChange) {
      onVisibleChange(true);
    }

    if (!hasVisibleFromProps) {
      setVisibleFromSelf(true);
    }
  }, [hasVisibleFromProps, onVisibleChange]);
  var close = useCallback(function () {
    if (onVisibleChange) {
      onVisibleChange(false);
    }

    if (!hasVisibleFromProps) {
      setVisibleFromSelf(false);
    }
  }, [hasVisibleFromProps, onVisibleChange]);
  var handleOpen = useCallback(function () {
    if (cancelLeaveDebounce.current) {
      cancelLeaveDebounce.current();
      cancelLeaveDebounce.current = null;
    }

    if (mouseEnterDelay === 0) {
      open();
    } else {
      var debounced = debounce(open, mouseEnterDelay);
      debounced();
      cancelEnterDebounce.current = debounced.cancel;
    }
  }, [mouseEnterDelay, open]);
  var handleClose = useCallback(function () {
    if (cancelEnterDebounce.current) {
      cancelEnterDebounce.current();
      cancelEnterDebounce.current = null;
    }

    if (mouseLeaveDelay === 0) {
      close();
    } else {
      var debounced = debounce(close, mouseLeaveDelay);
      debounced();
      cancelLeaveDebounce.current = debounced.cancel;
    }
  }, [close, mouseLeaveDelay]);
  return {
    handleOpen: handleOpen,
    handleClose: handleClose,
    visible: visible
  };
};

export { useHoverTrigger };