import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { forwardRef, useContext, useRef } from 'react';
import { useCombobox, useMultipleSelection } from 'downshift';
import { ClickOutsideContext } from '../../Popover';
import { Flex } from '../../Layout';
import { Tag } from '../../Tag';
import { useFormField } from '../../FormField';
import SelectOptions from './SelectOptions';
import SelectSuffix from './SelectSuffix';
import { StyledSelect } from './styles';
import { isCreateOption, itemToString, useIgnoreDownshiftUselessWarning, useSelectOptions } from './utils';
var MultipleSelect = /*#__PURE__*/forwardRef(function MultipleSelect(_ref, ref) {
  var id = _ref.id,
      _ref$loading = _ref.loading,
      loading = _ref$loading === void 0 ? false : _ref$loading,
      _ref$searchable = _ref.searchable,
      searchable = _ref$searchable === void 0 ? false : _ref$searchable,
      _ref$clearable = _ref.clearable,
      clearable = _ref$clearable === void 0 ? false : _ref$clearable,
      _ref$creatable = _ref.creatable,
      creatable = _ref$creatable === void 0 ? false : _ref$creatable,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      _ref$width = _ref.width,
      width = _ref$width === void 0 ? 240 : _ref$width,
      options = _ref.options,
      value = _ref.value,
      defaultValue = _ref.defaultValue,
      onChange = _ref.onChange,
      placeholder = _ref.placeholder,
      noOptionsMessage = _ref.noOptionsMessage,
      onCreateOption = _ref.onCreateOption,
      _ref$formatCreateLabe = _ref.formatCreateLabel,
      formatCreateLabel = _ref$formatCreateLabe === void 0 ? function (info) {
    return "Create new option: ".concat(info.value);
  } : _ref$formatCreateLabe,
      _ref$isValidNewOption = _ref.isValidNewOption,
      isValidNewOption = _ref$isValidNewOption === void 0 ? function (optionValue) {
    return optionValue.trim() !== '';
  } : _ref$isValidNewOption;
  var inputRef = useRef(null);

  var _useFormField = useFormField({
    id: id,
    value: value,
    defaultValue: defaultValue
  }),
      _useFormField2 = _slicedToArray(_useFormField, 3),
      invalid = _useFormField2[0],
      labelId = _useFormField2[1],
      setValue = _useFormField2[2];

  var _useContext = useContext(ClickOutsideContext),
      setHasChild = _useContext.setHasChild;

  var selectRef = useRef(null);

  var _useSelectOptions = useSelectOptions({
    options: options,
    creatable: creatable,
    isValidNewOption: isValidNewOption
  }),
      searchValue = _useSelectOptions.searchValue,
      setSearchValue = _useSelectOptions.setSearchValue,
      selectOptions = _useSelectOptions.selectOptions;

  useIgnoreDownshiftUselessWarning();

  var handleChange = function handleChange(_ref2) {
    var selectedItems = _ref2.selectedItems;
    var selection = selectedItems !== null && selectedItems !== void 0 ? selectedItems : [];
    var createOption = selection.find(isCreateOption);

    if (createOption && onCreateOption) {
      setSearchValue('');
      onCreateOption(createOption.value);
    } else {
      onChange(selectedItems);
      setValue(selectedItems);
    }
  };

  var _useMultipleSelection = useMultipleSelection({
    onSelectedItemsChange: handleChange,
    selectedItems: value,
    initialSelectedItems: defaultValue
  }),
      getSelectedItemProps = _useMultipleSelection.getSelectedItemProps,
      getDropdownProps = _useMultipleSelection.getDropdownProps,
      selectedItems = _useMultipleSelection.selectedItems,
      setSelectedItems = _useMultipleSelection.setSelectedItems;

  var _useCombobox = useCombobox({
    id: labelId,
    itemToString: itemToString,
    items: selectOptions,
    inputValue: searchValue,
    scrollIntoView: function scrollIntoView() {},
    onIsOpenChange: function onIsOpenChange(_ref3) {
      var visible = _ref3.isOpen;
      return setHasChild(Boolean(visible));
    },
    stateReducer: function stateReducer(state, actionAndChanges) {
      var changes = actionAndChanges.changes,
          type = actionAndChanges.type;

      if (type === useCombobox.stateChangeTypes.InputKeyDownEnter || type === useCombobox.stateChangeTypes.ItemClick) {
        setSearchValue('');
        var selectedItem = changes.selectedItem;
        var selectedItemString = itemToString(selectedItem);

        if (selectedItems.map(itemToString).includes(selectedItemString)) {
          setSelectedItems(selectedItems.filter(function (item) {
            return itemToString(item) !== selectedItemString;
          }));
        } else {
          setSelectedItems([].concat(_toConsumableArray(selectedItems), [selectedItem]));
        }

        var newHighlightedIndex = options.findIndex(function (item) {
          return itemToString(item) === selectedItemString;
        });
        return _objectSpread(_objectSpread({}, changes), {}, {
          highlightedIndex: newHighlightedIndex,
          isOpen: true // keep the menu open after selection.

        });
      }

      return changes;
    }
  }),
      highlightedIndex = _useCombobox.highlightedIndex,
      getToggleButtonProps = _useCombobox.getToggleButtonProps,
      getMenuProps = _useCombobox.getMenuProps,
      getInputProps = _useCombobox.getInputProps,
      getComboboxProps = _useCombobox.getComboboxProps,
      getItemProps = _useCombobox.getItemProps,
      isOpen = _useCombobox.isOpen;

  return /*#__PURE__*/React.createElement("div", _objectSpread({}, getComboboxProps({
    ref: ref,
    style: {
      width: width
    }
  })), /*#__PURE__*/React.createElement(StyledSelect, _objectSpread(_objectSpread({}, getToggleButtonProps({
    ref: selectRef,
    disabled: disabled || loading
  })), {}, {
    multiple: true,
    invalid: invalid,
    focused: isOpen
  }), /*#__PURE__*/React.createElement(Flex, {
    flex: "auto",
    flexWrap: "wrap",
    width: "calc(100% - 26px)"
  }, selectedItems.map(function (selectedItem, index) {
    return /*#__PURE__*/React.createElement(Tag, _objectSpread({
      key: itemToString(selectedItem),
      closable: true,
      disabled: loading || disabled,
      style: {
        marginBottom: 2,
        maxWidth: '100%'
      },
      onClose: function onClose() {
        var _inputRef$current;

        setSelectedItems(selectedItems.filter(function (item) {
          return itemToString(item) !== itemToString(selectedItem);
        }));
        (_inputRef$current = inputRef.current) === null || _inputRef$current === void 0 ? void 0 : _inputRef$current.focus();
      }
    }, getSelectedItemProps({
      selectedItem: selectedItem,
      index: index
    })), itemToString(selectedItem));
  }), /*#__PURE__*/React.createElement("input", _objectSpread({}, getInputProps(_objectSpread(_objectSpread({}, getDropdownProps({
    ref: inputRef,
    preventKeyAction: isOpen,
    readOnly: !searchable && !creatable || loading,
    placeholder: placeholder
  })), {}, {
    onChange: function onChange(event) {
      return setSearchValue(event.currentTarget.value);
    }
  }))))), /*#__PURE__*/React.createElement(SelectSuffix, {
    loading: loading,
    hasSelectedItem: selectedItems.length > 0,
    clearable: clearable,
    onClearIconClick: function onClearIconClick() {
      return setSelectedItems([]);
    },
    visible: isOpen
  })), /*#__PURE__*/React.createElement(SelectOptions, {
    selectRef: selectRef,
    getMenuProps: getMenuProps,
    getItemProps: getItemProps,
    visible: isOpen,
    options: selectOptions,
    selectedItems: selectedItems,
    highlightedIndex: highlightedIndex,
    formatCreateLabel: formatCreateLabel,
    noOptionsMessage: noOptionsMessage
  }));
});
export { MultipleSelect };