import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { forwardRef, useContext, useRef } from 'react';
import { useCombobox } from 'downshift';
import { ClickOutsideContext } from '../../Popover';
import { useFormField } from '../../FormField';
import SelectOptions from './SelectOptions';
import SelectSuffix from './SelectSuffix';
import { CREATE_OPTION, isCreateOption, itemToString, useIgnoreDownshiftUselessWarning, useSelectOptions } from './utils';
import { StyledSelect } from './styles';
var Select = /*#__PURE__*/forwardRef(function Select(_ref, ref) {
  var id = _ref.id,
      _ref$loading = _ref.loading,
      loading = _ref$loading === void 0 ? false : _ref$loading,
      _ref$searchable = _ref.searchable,
      searchable = _ref$searchable === void 0 ? false : _ref$searchable,
      _ref$clearable = _ref.clearable,
      clearable = _ref$clearable === void 0 ? false : _ref$clearable,
      _ref$creatable = _ref.creatable,
      creatable = _ref$creatable === void 0 ? false : _ref$creatable,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      _ref$width = _ref.width,
      width = _ref$width === void 0 ? 240 : _ref$width,
      options = _ref.options,
      value = _ref.value,
      defaultValue = _ref.defaultValue,
      onChange = _ref.onChange,
      onSearch = _ref.onSearch,
      placeholder = _ref.placeholder,
      noOptionsMessage = _ref.noOptionsMessage,
      onCreateOption = _ref.onCreateOption,
      _ref$formatCreateLabe = _ref.formatCreateLabel,
      formatCreateLabel = _ref$formatCreateLabe === void 0 ? function (info) {
    return "Create new option: ".concat(info.value);
  } : _ref$formatCreateLabe,
      _ref$isValidNewOption = _ref.isValidNewOption,
      isValidNewOption = _ref$isValidNewOption === void 0 ? function (optionValue) {
    return optionValue.trim() !== '';
  } : _ref$isValidNewOption;

  var _useFormField = useFormField({
    id: id,
    value: value,
    defaultValue: defaultValue
  }),
      _useFormField2 = _slicedToArray(_useFormField, 3),
      invalid = _useFormField2[0],
      labelId = _useFormField2[1],
      setValue = _useFormField2[2];

  var _useContext = useContext(ClickOutsideContext),
      setHasChild = _useContext.setHasChild;

  var selectRef = useRef(null);

  var _useSelectOptions = useSelectOptions({
    onSearch: onSearch,
    options: options,
    creatable: creatable,
    isValidNewOption: isValidNewOption
  }),
      searchValue = _useSelectOptions.searchValue,
      setSearchValue = _useSelectOptions.setSearchValue,
      selectOptions = _useSelectOptions.selectOptions;

  useIgnoreDownshiftUselessWarning();

  var handleChange = function handleChange(_ref2) {
    var selectedItem = _ref2.selectedItem;
    var selection = selectedItem !== null && selectedItem !== void 0 ? selectedItem : null;

    if (isCreateOption(selection)) {
      if (onCreateOption) {
        setSearchValue('');
        onChange(null);
        onCreateOption(selection.value);
      }
    } else {
      onChange(selection);
      setValue(selection);
    }
  };

  var getDefaultHighlightIndex = function getDefaultHighlightIndex() {
    if (searchable && searchValue !== '') {
      return 0;
    }

    var index = selectOptions.findIndex(function (item) {
      return itemToString(item) === itemToString(value !== null && value !== void 0 ? value : defaultValue);
    });
    return index !== -1 ? index : 0;
  };

  var _useCombobox = useCombobox({
    id: labelId,
    itemToString: itemToString,
    items: selectOptions,
    inputValue: searchValue,
    selectedItem: value,
    initialSelectedItem: defaultValue,
    defaultHighlightedIndex: getDefaultHighlightIndex(),
    scrollIntoView: function scrollIntoView() {},
    onSelectedItemChange: handleChange,
    onIsOpenChange: function onIsOpenChange(_ref3) {
      var visible = _ref3.isOpen;
      setHasChild(Boolean(visible));

      if (visible) {
        setSearchValue('');
      }
    }
  }),
      selectedItem = _useCombobox.selectedItem,
      highlightedIndex = _useCombobox.highlightedIndex,
      getToggleButtonProps = _useCombobox.getToggleButtonProps,
      getMenuProps = _useCombobox.getMenuProps,
      getInputProps = _useCombobox.getInputProps,
      getComboboxProps = _useCombobox.getComboboxProps,
      getItemProps = _useCombobox.getItemProps,
      selectItem = _useCombobox.selectItem,
      isOpen = _useCombobox.isOpen;

  var selectedItemString = itemToString(selectedItem);
  return /*#__PURE__*/React.createElement("div", _objectSpread({}, getComboboxProps({
    ref: ref,
    style: {
      width: width
    }
  })), /*#__PURE__*/React.createElement(StyledSelect, _objectSpread(_objectSpread({}, getToggleButtonProps({
    ref: selectRef,
    disabled: disabled || loading
  })), {}, {
    invalid: invalid,
    focused: isOpen
  }), /*#__PURE__*/React.createElement("input", _objectSpread({}, getInputProps(_objectSpread(_objectSpread({}, (searchable || creatable) && isOpen ? {} : {
    value: selectedItemString
  }), {}, {
    readOnly: !searchable && !creatable || loading,
    placeholder: placeholder !== null && placeholder !== void 0 ? placeholder : selectedItemString,
    onChange: function onChange(event) {
      var newInputValue = event.currentTarget.value;

      if (isOpen && (searchable || creatable)) {
        if (newInputValue === CREATE_OPTION) {
          setSearchValue('');
        } else {
          setSearchValue(newInputValue);
        }
      }
    }
  })))), /*#__PURE__*/React.createElement(SelectSuffix, {
    loading: loading,
    hasSelectedItem: selectedItem !== null,
    clearable: clearable,
    onClearIconClick: function onClearIconClick() {
      return selectItem(null);
    },
    visible: isOpen
  })), /*#__PURE__*/React.createElement(SelectOptions, {
    selectRef: selectRef,
    getMenuProps: getMenuProps,
    getItemProps: getItemProps,
    visible: isOpen,
    options: selectOptions,
    selectedItem: selectedItem,
    highlightedIndex: highlightedIndex,
    formatCreateLabel: formatCreateLabel,
    noOptionsMessage: noOptionsMessage
  }));
});
export { Select };