import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _pt from "prop-types";
import Popover from '@reach/popover';
import React, { useCallback, useEffect, useRef } from 'react';
import { MdCheck, MdHighlightOff } from 'react-icons/md';
import { useVirtual } from 'react-virtual';
import { Box, Flex } from '../../Layout';
import { Ellipsis } from '../../Ellipsis';
import { Heading } from '../../Typography';
import { Icon } from '../../Icon';
import { Stack } from '../../Stack';
import { Tooltip } from '../../Tooltip';
import { useLocale } from '../../locale';
import { StyledPopover, StyledSelectOption } from './styles';
import { isCreateOption, isObjectOption, itemToString, positionMatchWidth } from './utils';

var DefaultNoOptionsMessage = function DefaultNoOptionsMessage() {
  var _useLocale = useLocale(),
      locale = _useLocale.locale;

  return /*#__PURE__*/React.createElement(Flex, {
    py: "3",
    alignItems: "center",
    justifyContent: "center",
    style: {
      cursor: 'not-allowed'
    }
  }, /*#__PURE__*/React.createElement(Icon, {
    type: MdHighlightOff,
    mr: "2"
  }), /*#__PURE__*/React.createElement(Heading.H5, {
    color: "gray500"
  }, locale.Select.noDataText));
};

var SelectOptions = function SelectOptions(_ref) {
  var selectRef = _ref.selectRef,
      getMenuProps = _ref.getMenuProps,
      getItemProps = _ref.getItemProps,
      visible = _ref.visible,
      options = _ref.options,
      selectedItem = _ref.selectedItem,
      selectedItems = _ref.selectedItems,
      highlightedIndex = _ref.highlightedIndex,
      formatCreateLabel = _ref.formatCreateLabel,
      _ref$noOptionsMessage = _ref.noOptionsMessage,
      noOptionsMessage = _ref$noOptionsMessage === void 0 ? function () {
    return /*#__PURE__*/React.createElement(DefaultNoOptionsMessage, null);
  } : _ref$noOptionsMessage;
  var listRef = useRef(null);

  var _useVirtual = useVirtual({
    size: options.length,
    parentRef: listRef,
    estimateSize: useCallback(function () {
      return 36;
    }, []),
    overscan: 3
  }),
      virtualItems = _useVirtual.virtualItems,
      totalSize = _useVirtual.totalSize,
      scrollToIndex = _useVirtual.scrollToIndex;

  useEffect(function () {
    if (visible && highlightedIndex !== -1) {
      scrollToIndex(highlightedIndex);
    }
  }, [visible, scrollToIndex, highlightedIndex]);
  return /*#__PURE__*/React.createElement(Stack, null, function (stackingOrder) {
    return /*#__PURE__*/React.createElement(Popover, {
      targetRef: selectRef,
      hidden: !visible,
      position: positionMatchWidth,
      style: {
        zIndex: stackingOrder
      }
    }, /*#__PURE__*/React.createElement(StyledPopover, _objectSpread({}, getMenuProps({
      ref: listRef
    })), visible && /*#__PURE__*/React.createElement(React.Fragment, null, virtualItems.length === 0 && /*#__PURE__*/React.createElement(Box, {
      bg: "light"
    }, noOptionsMessage()), /*#__PURE__*/React.createElement("div", {
      style: {
        height: totalSize
      }
    }, virtualItems.map(function (virtualRow) {
      var _ref2;

      var item = options[virtualRow.index];
      var itemString = itemToString(item);
      var itemDisabled = (_ref2 = isObjectOption(item) && item.disabled) !== null && _ref2 !== void 0 ? _ref2 : false;
      var hovered = highlightedIndex === virtualRow.index;
      var active = selectedItems ? selectedItems.map(itemToString).includes(itemString) : itemToString(selectedItem) === itemString;
      var content = isCreateOption(item) ? formatCreateLabel({
        value: item.value,
        active: active,
        hovered: hovered
      }) : itemString;
      var hint = isObjectOption(item) ? item.hint : undefined;
      return /*#__PURE__*/React.createElement(StyledSelectOption, _objectSpread({
        key: itemString,
        hovered: hovered,
        active: active
      }, getItemProps({
        item: item,
        index: virtualRow.index,
        disabled: itemDisabled,
        style: {
          height: virtualRow.size,
          transform: "translateY(".concat(virtualRow.start, "px)")
        }
      })), selectedItems && /*#__PURE__*/React.createElement(Box, {
        display: "inline-flex",
        flex: "none",
        width: "16px",
        mr: "8px"
      }, active && /*#__PURE__*/React.createElement(Icon, {
        type: MdCheck,
        fill: hovered ? 'light' : 'primaryLight',
        size: "16px"
      })), hint ? /*#__PURE__*/React.createElement(Tooltip, {
        content: hint,
        mouseLeaveDelay: 0
      }, /*#__PURE__*/React.createElement("div", {
        style: {
          width: '100%'
        }
      }, content)) : /*#__PURE__*/React.createElement(Ellipsis, null, content));
    })))));
  });
};

SelectOptions.propTypes = {
  visible: _pt.bool.isRequired,
  getMenuProps: _pt.any.isRequired,
  getItemProps: _pt.any.isRequired,
  options: _pt.array.isRequired,
  selectedItems: _pt.array,
  highlightedIndex: _pt.number.isRequired,
  formatCreateLabel: _pt.func.isRequired,
  noOptionsMessage: _pt.func
};
export default SelectOptions;