import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _typeof from "@babel/runtime/helpers/typeof";
import fuzzaldrin from 'fuzzaldrin-plus';
import React, { useEffect, useState } from 'react';
import { useIsomorphicLayoutEffect } from '@reach/utils';
export var CREATE_OPTION = 'CREATE_OPTION';
export var isObjectOption = function isObjectOption(item) {
  return _typeof(item) === 'object' && Boolean(item);
};
export var isCreateOption = function isCreateOption(item) {
  return isObjectOption(item) && item.label === CREATE_OPTION;
};
export var itemToString = function itemToString(item) {
  if (isObjectOption(item)) {
    return item.label;
  }

  if (item === null || typeof item === 'undefined') {
    return '';
  }

  return String(item);
};
export var filter = function filter(items, input) {
  var wrappedItems = items.map(function (item) {
    return {
      key: itemToString(item),
      item: item
    };
  });
  return fuzzaldrin.filter(wrappedItems, input, {
    key: 'key'
  }).map(function (_ref) {
    var item = _ref.item;
    return item;
  });
};
export var useSelectOptions = function useSelectOptions(_ref2) {
  var onSearch = _ref2.onSearch,
      options = _ref2.options,
      creatable = _ref2.creatable,
      isValidNewOption = _ref2.isValidNewOption;

  var _useState = useState(''),
      _useState2 = _slicedToArray(_useState, 2),
      searchValue = _useState2[0],
      setSearchValue = _useState2[1];

  var _useState3 = useState(options),
      _useState4 = _slicedToArray(_useState3, 2),
      selectOptions = _useState4[0],
      setSelectOptions = _useState4[1];

  useIsomorphicLayoutEffect(function () {
    if (onSearch && searchValue.trim() !== '') {
      onSearch(searchValue);
    }
  }, [searchValue]);
  useIsomorphicLayoutEffect(function () {
    var searchOptions = options;

    if (!onSearch) {
      searchOptions = searchValue !== '' ? filter(options, searchValue) : options;
    }

    var additionOptions = creatable && isValidNewOption(searchValue) ? [{
      label: CREATE_OPTION,
      value: searchValue
    }] : [];
    setSelectOptions([].concat(_toConsumableArray(searchOptions), additionOptions));
  }, [searchValue, options]);
  return {
    searchValue: searchValue,
    setSearchValue: setSearchValue,
    selectOptions: selectOptions
  };
};
export var useIgnoreDownshiftUselessWarning = function useIgnoreDownshiftUselessWarning() {
  useEffect(function () {
    var originalError = console.error;

    console.error = function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      if (/downshift: The ref prop "ref" from/.test(args[0])) {
        return;
      }

      originalError.call.apply(originalError, [console].concat(args));
    };

    return function () {
      console.error = originalError;
    };
  }, []);
};

function getCollisions(targetRect, popoverRect) {
  var offsetLeft = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var offsetBottom = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
  var collisions = {
    top: targetRect.top - popoverRect.height < 0,
    right: window.innerWidth < targetRect.left + popoverRect.width - offsetLeft,
    bottom: window.innerHeight < targetRect.bottom + popoverRect.height - offsetBottom,
    left: targetRect.left + targetRect.width - popoverRect.width < 0
  };
  var directionRight = collisions.right && !collisions.left;
  var directionLeft = collisions.left && !collisions.right;
  var directionUp = collisions.bottom && !collisions.top;
  var directionDown = collisions.top && !collisions.bottom;
  return {
    directionRight: directionRight,
    directionLeft: directionLeft,
    directionUp: directionUp,
    directionDown: directionDown
  };
}

var POSITION_Y_OFFSET = 5;

function getTopPosition(targetRect, popoverRect) {
  var _getCollisions = getCollisions(targetRect, popoverRect),
      directionUp = _getCollisions.directionUp;

  return {
    top: directionUp ? "".concat(targetRect.top - popoverRect.height + window.pageYOffset - POSITION_Y_OFFSET, "px") : "".concat(targetRect.top + targetRect.height + window.pageYOffset + POSITION_Y_OFFSET, "px")
  };
}

export var positionMatchWidth = function positionMatchWidth(targetRect, popoverRect) {
  if (!targetRect || !popoverRect) {
    return {};
  }

  return _objectSpread({
    width: targetRect.width,
    left: targetRect.left
  }, getTopPosition(targetRect, popoverRect));
};