'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var polished = require('polished');
var styledNormalize = require('styled-normalize');
var styledComponents = require('styled-components');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e['default'] : e; }

var styledNormalize__default = /*#__PURE__*/_interopDefaultLegacy(styledNormalize);

var colors = {
  primary: '#328ef5',
  primaryLight: '#3990cd',
  primaryLight2: '#a2d3ff',
  primaryDark: '#1d70b8',
  primaryDark2: '#0b4278',
  secondary: '#3dc8ff',
  secondaryLight: '#50acff',
  secondaryDark: '#0097cc',
  surface: '#eef1f8',
  surface2: '#fbfbfd',
  dark: '#000000',
  light: '#ffffff',
  gray100: '#f6f8fb',
  gray200: '#dee5ef',
  gray300: '#c4cfdd',
  gray400: '#97a4ba',
  gray500: '#67758d',
  gray600: '#424b5f',
  gray700: '#283040',
  gray800: '#191d28',
  success: '#51c81c',
  danger: '#ff5757',
  warning: '#fcb41d',
  info: '#328ef5',
  error: '#ff5757'
};
var darkerColors = {
  // primary
  primary: '#3c5ad0',
  primaryDark: '#184480',
  primaryDark2: '#001e40',
  primaryLight: '#6B88c5',
  primaryLight2: '#d5d8e8',
  // secondary
  secondary: '#50d5ff',
  secondaryLight: '#63a9f5',
  secondaryDark: '#2db8de',
  // surface
  surface: '#e8ecf5',
  surface2: '#f8f8fc',
  dark: '#000000',
  light: '#ffffff',
  // gray
  gray100: '#fafafa',
  gray200: '#f6f6f6',
  gray300: '#e0e0e0',
  gray400: '#bfbfbf',
  gray500: '#8b8b8b',
  gray600: '#5e5e5e',
  gray700: '#373737',
  gray800: '#1a1a1a',
  // status
  success: '#40d064',
  danger: '#fc5475',
  warning: '#f6a700',
  info: '#4384f0',
  error: '#fc5475' // FIXME: remove this line later

};

var toRem = function toRem(px) {
  return polished.rem("".concat(px, "px"));
};

var breakpoints = [toRem(576), toRem(768), toRem(992), toRem(1200)];
var space = {
  0: toRem(0),
  1: toRem(4),
  2: toRem(8),
  3: toRem(16),
  4: toRem(32),
  5: toRem(64),
  6: toRem(128),
  7: toRem(256)
};
var paddings = {
  xs: polished.rem('8px'),
  sm: polished.rem('12px'),
  md: polished.rem('16px'),
  lg: polished.rem('24px')
};
var heights = {
  sm: polished.rem('28px'),
  base: polished.rem('36px'),
  lg: polished.rem('44px')
};
var fontSizes = {
  sm: polished.rem('14px'),
  base: polished.rem('16px'),
  lg: polished.rem('18px')
};
// border-radius
var radii = {
  sm: polished.rem('2px'),
  base: polished.rem('4px'),
  lg: polished.rem('8px'),
  xl: polished.rem('16px')
};
var borders = {
  base: '1px solid',
  lg: '2px solid',
  dashed: '1px dashed'
};
var transition = 'transition: all 200ms ease 0s';
var shadows = {
  sm: '0 1px 3px 0 #C4CFDD',
  base: '0 2px 4px 0 #C4CFDD',
  lg: '0 2px 6px 0 #C4CFDD',
  xl: '0 5px 10px 0 #C4CFDD'
};
var theme = {
  breakpoints: breakpoints,
  colors: colors,
  space: space,
  paddings: paddings,
  heights: heights,
  fontSizes: fontSizes,
  radii: radii,
  borders: borders,
  shadows: shadows,
  lineHeight: 1.5,
  transition: transition
};
var darkerTheme = {
  breakpoints: breakpoints,
  colors: darkerColors,
  space: space,
  paddings: paddings,
  heights: heights,
  fontSizes: fontSizes,
  radii: radii,
  borders: borders,
  shadows: shadows,
  lineHeight: 1.5,
  transition: transition
};

var globalStyle = styledComponents.css(["", " *,*::before,*::after{box-sizing:inherit;}html{box-sizing:border-box;font-size:16px;line-height:1.5;-webkit-tap-highlight-color:transparent;}body{overflow-x:hidden;background:", ";color:", ";font-family:Roboto,'PingFang TC','Microsoft JhengHei','Helvetica Neue',sans-serif;font-size:", ";-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;text-rendering:optimizeLegibility;}a{color:", ";text-decoration:none;cursor:pointer;}"], styledNormalize__default, function (p) {
  return p.theme.colors.light;
}, function (p) {
  return p.theme.colors.gray700;
}, polished.rem('16px'), function (p) {
  return p.theme.colors.info;
});

var fontStyle = "\n  @import url('https://fonts.googleapis.com/css?family=Roboto');\n  @import url('https://fonts.googleapis.com/css?family=Roboto+Mono');\n";

exports.darkerTheme = darkerTheme;
exports.fontStyle = fontStyle;
exports.globalStyle = globalStyle;
exports.theme = theme;
//# sourceMappingURL=index.cjs.js.map
